@@ -26,11 +26,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 19279 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-01-17 05:08:34 +0100 (jeu 17 jan 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
+#ifndef OPAL_H323_H323_H
+#define OPAL_H323_H323_H
+
+#include <opal/buildopts.h>
+
 #include <h323/h323ep.h>
 #include <h323/h323con.h>
 #include <h323/gkclient.h>
@@ -42,4 +47,6 @@ unsigned OpalGetMinorVersion();
 unsigned OpalGetBuildNumber();
 
 
+#endif // OPAL_H323_H323_H
+
 /////////////////////////////////////////////////////////////////////////////
