@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21765 $
+ * $Author: hfriederich $
+ * $Date: 2008-12-10 07:59:41 +0000 (Wed, 10 Dec 2008) $
  */
 
 #ifndef OPAL_H323_H323CON_H
@@ -1346,6 +1346,15 @@ class H323Connection : public OpalRTPConnection
                                          ///<  Parameters for channel
       RTP_QOS * rtpqos = NULL            ///<  QoS for RTP
     );
+    
+    /**Creates a new instance of an RTP channel.
+       Allows subclasses to return a custom instance
+      */
+    virtual H323_RTPChannel * CreateRTPChannel(
+      const H323Capability & capability,
+      H323Channel::Directions direction,
+      RTP_Session & rtp
+    );
 
     /**This function is called when the remote endpoint want's to create
        a new channel.
@@ -1896,26 +1905,13 @@ class H323Connection : public OpalRTPConnection
     H4507Handler&  getH4507handler(){return *h4507handler;}
 #endif
 
-   /**Open the media channels associated with an incoming call
- 
-       This function is provided to allow an OpalConnection descendant to
-       delay the opening of the media channels associated with an incoming
-       call. By default, this function is called as soon as possible after an
-       incoming connection request has been received (i.e. an INVITE or SETUP)
-       and and do all of the work associated with handling SDP or fastStart
-       eyc
-
-       By overriding this function, the media channel open is deferred, which
-       gives the other connections in the call a chance to do something before
-       the media channels are started. This allows for features beyond those
-       provided by the OnAnswer interface
-
-       Overriding this function is ok, provided the application ensures that
-       this method in this class is called.
-      */
-    virtual PBoolean OnOpenIncomingMediaChannels();
-
     virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    
+    // H.245 - compliant handling of session IDs
+    // if no internal session ID is found, zero is returned
+    unsigned GetExternalSessionID(unsigned internalSessionID) const;
+    unsigned GetExternalSessionID(unsigned internalSessionID, const OpalMediaType & mediaType);
+    unsigned GetInternalSessionID(unsigned externalSessionID, const OpalMediaType & mediaType);
 
   protected:
     /**Internal function to check if call established.
@@ -2011,6 +2007,15 @@ class H323Connection : public OpalRTPConnection
     FastStartStates        fastStartState;
     H323LogicalChannelList fastStartChannels;
     OpalMediaStreamPtr     fastStartMediaStream;
+    
+    // H.245 - compliant handling of session IDs
+    PBoolean defaultAudioSessionIDAssigned, defaultVideoSessionIDAssigned;
+    unsigned nextSessionID;
+    typedef std::map<unsigned, unsigned> InternalToExternalSessionIDMap;
+    typedef std::map<OpalMediaType, unsigned> MediaTypeToInternalSessionIDMap;
+    InternalToExternalSessionIDMap internalToExternalSessionIDMap;
+    MediaTypeToInternalSessionIDMap mediaTypeToInternalSessionIDMap;
+    PMutex sessionIDHandlingMutex;
 
 #if PTRACING
     static const char * GetConnectionStatesName(ConnectionStates s);
