@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21317 $
+ * $Author: dereksmithies $
+ * $Date: 2008-10-13 21:35:37 +0000 (Mon, 13 Oct 2008) $
  */
 
 #ifndef OPAL_IAX2_IAX2EP_H
@@ -289,8 +289,6 @@ class IAX2EndPoint : public OpalEndPoint
       */
   virtual OpalMediaFormatList GetMediaFormats() const;
 
-  virtual PBoolean OnIncomingCall(IAX2Connection & conn);
-
   /**Return the bitmask which specifies the possible codecs we support */
   PINDEX GetSupportedCodecs(OpalMediaFormatList & list);
   
