@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 21816 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2008-12-17 02:54:09 +0000 (Wed, 17 Dec 2008) $
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -631,7 +631,9 @@ class OpalEndPoint : public PObject
 
     /**Set the default local party name for all connections on this endpoint.
       */
-    void SetDefaultLocalPartyName(const PString & name) { defaultLocalPartyName = name; }
+    virtual void SetDefaultLocalPartyName(
+      const PString & name  /// Name for local party
+    ) { defaultLocalPartyName = name; }
 
     /**Get the default local display name for all connections on this endpoint.
       */
@@ -653,6 +655,18 @@ class OpalEndPoint : public PObject
      */
     const OpalListenerList & GetListeners() const { return listeners; }
 
+    /**Get the default options for created connections.
+      */
+    const OpalConnection::StringOptions & GetDefaultStringOptions() const { return m_defaultStringOptions; }
+
+    /**Set the default options for created connections.
+      */
+    void SetDefaultStringOptions(const OpalConnection::StringOptions & opts) { m_defaultStringOptions = opts; }
+
+    /**Set the default option for created connections.
+      */
+    void SetDefaultStringOption(const PCaselessString & key, const PString & data) { m_defaultStringOptions.SetAt(key, data); }
+
     /**Get the default mode for sending User Input Indications.
       */
     OpalConnection::SendUserInputModes GetSendUserInputMode() const { return defaultSendUserInputMode; }
@@ -678,6 +692,18 @@ class OpalEndPoint : public PObject
     PString GetSSLCertificate() const;
 #endif
 
+    /** Find a listener that is compatible with the specified protocol
+     */
+    bool FindListenerForProtocol(const char * protoPrefix, OpalTransportAddress & addr);
+
+    /**Add IM media formats available on a connection.
+       The default behaviour calls the OpalEndPoint function of the same name.
+      */
+    virtual void AddIMMediaFormats(
+      OpalMediaFormatList & mediaFormats, ///<  Media formats to use
+      const OpalConnection * connection = NULL  ///<  Optional connection that is using formats
+    ) const;
+
   protected:
     OpalManager   & manager;
     PCaselessString prefixName;
@@ -688,6 +714,7 @@ class OpalEndPoint : public PObject
     PString         defaultDisplayName;
 
     unsigned initialBandwidth;  // in 100s of bits/sev
+    OpalConnection::StringOptions      m_defaultStringOptions;
     OpalConnection::SendUserInputModes defaultSendUserInputMode;
 
     OpalListenerList   listeners;
