@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
- * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Revision: 21848 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-22 00:47:09 +0000 (Mon, 22 Dec 2008) $
  */
 
 #ifndef OPAL_OPAL_MEDIAFMT_H
@@ -140,6 +140,18 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
       const_iterator start = const_iterator()
     ) const;
 
+    /**Get a format position in the list not matching the wildcard.
+       The wildcard string is a simple substring match using the '*'
+       character. For example: "G.711*" would match "G.711-uLaw-64k" and
+       "G.711-ALaw-64k".
+
+       Returns P_MAX_INDEX list contains only matches.
+      */
+    const_iterator FindNotFormat(
+      const PString & wildcard,    ///<  Wildcard string name.
+      const_iterator start = const_iterator()
+    ) const;
+
     /**Determine if a format matching the payload type is in the list.
       */
     PBoolean HasFormat(
@@ -168,6 +180,13 @@ class OpalMediaFormatList : public OpalMediaFormatBaseList
     void Reorder(
       const PStringArray & order
     );
+
+    /**Determine if a list contains a specific media type
+      */
+    bool HasType(
+      const OpalMediaType & type,
+      bool mustBeTransportable = true
+    ) const;
   //@}
 
   private:
@@ -510,8 +529,8 @@ class OpalMediaFormatInternal : public PObject
     virtual PObject * Clone() const;
     virtual void PrintOn(ostream & strm) const;
 
-    virtual PBoolean IsValid() const { return rtpPayloadType < RTP_DataFrame::IllegalPayloadType && !formatName.IsEmpty(); }
-    virtual PBoolean IsTransportable() const { return rtpPayloadType < RTP_DataFrame::MaxPayloadType && !rtpEncodingName.IsEmpty(); }
+    virtual bool IsValid() const;
+    virtual bool IsTransportable() const;
 
     virtual PStringToString GetOptions() const;
     virtual bool GetOptionValue(const PString & name, PString & value) const;
@@ -545,6 +564,7 @@ class OpalMediaFormatInternal : public PObject
     PMutex                       media_format_mutex;
     PSortedList<OpalMediaOption> options;
     time_t                       codecVersionTime;
+    bool                         forceIsTransportable;
 
   friend bool operator==(const char * other, const OpalMediaFormat & fmt);
   friend bool operator!=(const char * other, const OpalMediaFormat & fmt);
@@ -1011,6 +1031,7 @@ class OpalMediaFormat : public PContainer
 
 #if OPAL_H323
     static const PString & MediaPacketizationOption();
+    static const PString & MediaPacketizationsOption();
 #endif
 
   private:
@@ -1120,8 +1141,12 @@ class OpalVideoFormat : public OpalMediaFormat
 
 #define OPAL_PCM16          "PCM-16"
 #define OPAL_PCM16_16KHZ    "PCM-16-16kHz"
+#define OPAL_PCM16_32KHZ    "PCM-16-32kHz"
+#define OPAL_PCM16_48KHZ    "PCM-16-48kHz"
 #define OPAL_L16_MONO_8KHZ  "Linear-16-Mono-8kHz"
 #define OPAL_L16_MONO_16KHZ "Linear-16-Mono-16kHz"
+#define OPAL_L16_MONO_32KHZ "Linear-16-Mono-32kHz"
+#define OPAL_L16_MONO_48KHZ "Linear-16-Mono-48kHz"
 #define OPAL_G711_ULAW_64K  "G.711-uLaw-64k"
 #define OPAL_G711_ALAW_64K  "G.711-ALaw-64k"
 #define OPAL_G726_40K       "G.726-40K"
@@ -1141,15 +1166,22 @@ class OpalVideoFormat : public OpalMediaFormat
 #define OPAL_GSM0610        "GSM-06.10"
 #define OPAL_GSMAMR         "GSM-AMR"
 #define OPAL_iLBC           "iLBC"
+#define OPAL_H261           "H.261"
+#define OPAL_H263           "H.263"
+#define OPAL_H264           "H.264"
+#define OPAL_MPEG4          "MPEG4"
 #define OPAL_RFC2833        "UserInput/RFC2833"
 #define OPAL_CISCONSE       "NamedSignalEvent"
 #define OPAL_T38            "T.38"
-#define OPAL_H224            "H.224"
 
 extern const OpalAudioFormat & GetOpalPCM16();
 extern const OpalAudioFormat & GetOpalPCM16_16KHZ();
+extern const OpalAudioFormat & GetOpalPCM16_32KHZ();
+extern const OpalAudioFormat & GetOpalPCM16_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_8KHZ();
 extern const OpalAudioFormat & GetOpalL16_MONO_16KHZ();
+extern const OpalAudioFormat & GetOpalL16_MONO_32KHZ();
+extern const OpalAudioFormat & GetOpalL16_MONO_48KHZ();
 extern const OpalAudioFormat & GetOpalL16_STEREO_48KHZ();
 extern const OpalAudioFormat & GetOpalG711_ULAW_64K();
 extern const OpalAudioFormat & GetOpalG711_ALAW_64K();
@@ -1168,18 +1200,24 @@ extern const OpalAudioFormat & GetOpalG7231A_6k3();
 extern const OpalAudioFormat & GetOpalG7231A_5k3();
 extern const OpalAudioFormat & GetOpalGSM0610();
 extern const OpalAudioFormat & GetOpalGSMAMR();
-extern const OpalAudioFormat & GetOpaliLBC();
+extern const OpalMediaFormat & GetOpaliLBC();
 
 extern const OpalMediaFormat & GetOpalRFC2833();
+
 #if OPAL_T38_CAPABILITY
 extern const OpalMediaFormat & GetOpalCiscoNSE();
 extern const OpalMediaFormat & GetOpalT38();
 #endif
 
+
 #define OpalPCM16          GetOpalPCM16()
 #define OpalPCM16_16KHZ    GetOpalPCM16_16KHZ()
+#define OpalPCM16_32KHZ    GetOpalPCM16_32KHZ()
+#define OpalPCM16_48KHZ    GetOpalPCM16_48KHZ()
 #define OpalL16_MONO_8KHZ  GetOpalL16_MONO_8KHZ()
 #define OpalL16_MONO_16KHZ GetOpalL16_MONO_16KHZ()
+#define OpalL16_MONO_32KHZ GetOpalL16_MONO_32KHZ()
+#define OpalL16_MONO_48KHZ GetOpalL16_MONO_48KHZ()
 #define OpalL16_STEREO_48KHZ GetOpalL16_STEREO_48KHZ()
 #define OpalG711_ULAW_64K  GetOpalG711_ULAW_64K()
 #define OpalG711_ALAW_64K  GetOpalG711_ALAW_64K()
@@ -1208,6 +1246,22 @@ extern const OpalMediaFormat & GetOpalT38();
 #define OpalG711uLaw       OpalG711_ULAW_64K
 #define OpalG711ALaw       OpalG711_ALAW_64K
 
+#define OPAL_T140             "T.140"
+#define OpalT140              GetOpalT140()
+extern const OpalMediaFormat & GetOpalT140();
+
+#if OPAL_HAS_MSRP
+#define OPAL_MSRP           "MSRP"
+#define OpalMSRP            GetOpalMSRP()
+extern const OpalMediaFormat & GetOpalMSRP();
+#endif
+
+#if OPAL_HAS_SIPIM
+#define OPAL_SIPIM             "SIP-IM"
+#define OpalSIPIM              GetOpalSIPIM()
+extern const OpalMediaFormat & GetOpalSIPIM();
+#endif
+
 #ifdef _MSC_VER
 #if _MSC_VER < 1300
 #pragma warning(default:4663)
