@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-#ifndef __OPAL_MEDIASTRM_H
-#define __OPAL_MEDIASTRM_H
+#ifndef OPAL_OPAL_MEDIASTRM_H
+#define OPAL_OPAL_MEDIASTRM_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -887,7 +887,7 @@ class OpalUDPMediaStream : public OpalMediaStream
 };
 
 
-#endif //__OPAL_MEDIASTRM_H
+#endif //OPAL_OPAL_MEDIASTRM_H
 
 
 // End of File ///////////////////////////////////////////////////////////////
