@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21686 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-03 04:47:39 +0000 (Wed, 03 Dec 2008) $
  */
 
 #ifndef OPAL_OPAL_PCSS_H
@@ -124,6 +124,14 @@ class OpalPCSSEndPoint : public OpalEndPoint
       const PString & recordDevice, ///<  Sound channel record device
       void * userData     ///<  Arbitrary data to pass to connection
     );
+    virtual OpalPCSSConnection * CreateConnection(
+      OpalCall & call,    ///<  Owner of connection
+      const PString & playDevice, ///<  Sound channel play device
+      const PString & recordDevice, ///<  Sound channel record device
+      void * userData,    ///<  Arbitrary data to pass to connection
+      unsigned options,
+      OpalConnection::StringOptions * stringOptions
+    );
 
     /**Create an PSoundChannel based media stream.
       */
@@ -269,7 +277,9 @@ class OpalPCSSConnection : public OpalConnection
       OpalCall & call,              ///<  Owner calll for connection
       OpalPCSSEndPoint & endpoint,  ///<  Owner endpoint for connection
       const PString & playDevice,   ///<  Sound channel play device
-      const PString & recordDevice  ///<  Sound channel record device
+      const PString & recordDevice,  ///<  Sound channel record device
+      unsigned options = 0,
+      OpalConnection::StringOptions * stringOptions = NULL
     );
 
     /**Destroy endpoint.
