@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21283 $
+ * $Revision: 21811 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2008-12-16 10:06:55 +0000 (Tue, 16 Dec 2008) $
  */
 
 #ifndef OPAL_OPAL_RTPCONN_H
@@ -40,6 +40,24 @@
 #include <opal/connection.h>
 #include <opal/mediatype.h>
 
+#ifdef OPAL_ZRTP
+
+class OpalZRTPStreamInfo {
+  public:
+    virtual bool Open() = 0;
+    virtual RTP_UDP * CreateRTPSession(OpalConnection & conn, unsigned sessionId, bool remoteIsNat) = 0;
+};
+
+class OpalZRTPConnectionInfo {
+  public:
+    virtual bool Open() = 0;
+    virtual RTP_UDP * CreateRTPSession(OpalConnection & conn, unsigned sessionId, bool remoteIsNat) = 0;
+
+    PMutex mutex;
+};
+
+#endif // OPAL_ZRTP
+
 
 class OpalRTPEndPoint;
 
@@ -55,15 +73,72 @@ class OpalMediaSession : public PObject
 {
   PCLASSINFO(OpalMediaSession, PObject);
   public:
-    OpalMediaSession(const OpalMediaType & _mediaType);
+    OpalMediaSession(OpalConnection & conn, const OpalMediaType & _mediaType, unsigned sessionId);
     OpalMediaSession(const OpalMediaSession & _obj);
 
-    PObject * Clone() const { return new OpalMediaSession(*this); }
+    virtual void Close() = 0;
 
+    virtual PObject * Clone() const = 0;
+
+    virtual bool IsActive() const = 0;
+
+    virtual bool IsRTP() const = 0;
+
+    virtual bool HasFailed() const = 0;
+
+    virtual OpalTransportAddress GetLocalMediaAddress() const = 0;
+
+    virtual void SetRemoteMediaAddress(const OpalTransportAddress &, const OpalMediaFormatList & ) { }
+
+#if OPAL_SIP
+    virtual SDPMediaDescription * CreateSDPMediaDescription(
+      const OpalTransportAddress & localAddress
+    ) = 0;
+#endif
+
+    virtual OpalMediaStream * CreateMediaStream(
+      const OpalMediaFormat & mediaFormat, 
+      unsigned sessionID, 
+      PBoolean isSource
+    ) = 0;
+
+    OpalConnection & connection;
     OpalMediaType mediaType;     // media type for session
-    bool autoStartReceive;       // if true, this session should receive data when the call is started
-    bool autoStartTransmit;      // if true, this session  should transmit data when the call is started
     unsigned sessionId;          // unique session ID
+};
+
+/** Class for carrying RTP session information
+  */
+class OpalRTPMediaSession : public OpalMediaSession
+{
+  PCLASSINFO(OpalRTPMediaSession, OpalMediaSession);
+  public:
+    OpalRTPMediaSession(OpalConnection & conn, const OpalMediaType & mediaType, unsigned sessionId);
+    OpalRTPMediaSession(const OpalRTPMediaSession & obj);
+
+    PObject * Clone() const { return new OpalRTPMediaSession(*this); }
+
+    virtual void Close();
+
+    virtual bool IsActive() const { return rtpSession != NULL; }
+
+    virtual bool IsRTP() const { return true; }
+
+    virtual bool HasFailed() const { return rtpSession != NULL && rtpSession->HasFailed(); }
+
+    virtual OpalTransportAddress GetLocalMediaAddress() const;
+
+#if OPAL_SIP
+    virtual SDPMediaDescription * CreateSDPMediaDescription(
+      const OpalTransportAddress & localAddress
+    );
+#endif
+
+    virtual OpalMediaStream * CreateMediaStream(
+      const OpalMediaFormat & mediaFormat, 
+      unsigned sessionID, 
+      PBoolean isSource
+    );
 
     RTP_Session * rtpSession;    // RTP session
 };
@@ -79,31 +154,11 @@ class OpalRTPSessionManager : public PObject
   //@{
     /**Construct new session manager database.
       */
-    OpalRTPSessionManager();
+    OpalRTPSessionManager(OpalConnection & conn);
     ~OpalRTPSessionManager();
+    void operator=(const OpalRTPSessionManager & other) { sessions = other.sessions; }
   //@}
 
-    void CopyFromMaster(const OpalRTPSessionManager & sm);
-    void CopyToMaster(OpalRTPSessionManager & sm);
-
-    /**
-       Initialise the autostart options within the session
-      */
-    void Initialise(
-      OpalRTPConnection & conn, 
-      OpalConnection::StringOptions * stringOptions
-    );
-
-    /**
-       Initialise the autostart options for a session
-      */
-    unsigned AutoStartSession(
-      unsigned sessionID,        ///<  Session ID to use.
-      const OpalMediaType & mediaType,
-      bool autoStartReceive, 
-      bool autoStartTransmit
-    );
-
   /**@name Operations */
   //@{
     /**Add an RTP session for the specified ID.
@@ -116,6 +171,10 @@ class OpalRTPSessionManager : public PObject
       RTP_Session * session,          ///<  Session to add.
       const OpalMediaType & mediaType ///< initial media type for this session
     );
+    void AddMediaSession(
+      OpalMediaSession * session,          ///<  Session to add.
+      const OpalMediaType & mediaType ///< initial media type for this session
+    );
 
     /**Release the session.
      */
@@ -129,25 +188,21 @@ class OpalRTPSessionManager : public PObject
     RTP_Session * GetSession(
       unsigned sessionID    ///<  Session ID to get.
     ) const;
-
-    void SetCleanup(bool v) { m_cleanupOnDelete = v; }
-
+    OpalMediaSession * GetMediaSession(
+      unsigned sessionID
+    ) const;
   //@}
+
     PMutex & GetMutex() { return m_mutex; }
 
-  protected:
-    void SetOldOptions(unsigned channelId, const OpalMediaType & mediaType, bool rx, bool tx);
+    virtual bool AllSessionsFailing();
 
+  protected:
+    OpalConnection & connection;
     PMutex m_mutex;
-    bool m_initialised;
-    bool m_cleanupOnDelete;
+
     PDICTIONARY(SessionDict, POrdinalKey, OpalMediaSession);
     SessionDict sessions;
-
-  private:
-    OpalRTPSessionManager (const OpalRTPSessionManager & other): PObject (other) { }
-    OpalRTPSessionManager & operator=(const OpalRTPSessionManager &) { return *this; }
-
 };
 
 typedef OpalRTPSessionManager RTP_SessionManager;
@@ -184,6 +239,9 @@ class OpalRTPConnection : public OpalConnection
     virtual RTP_Session * GetSession(
       unsigned sessionID    ///<  RTP session number
     ) const;
+    virtual OpalMediaSession * GetMediaSession(
+      unsigned sessionID    ///<  RTP session number
+    ) const;
 
     /**Use an RTP session for the specified ID.
        This will find a session of the specified ID and uses it if available.
@@ -318,29 +376,26 @@ class OpalRTPConnection : public OpalConnection
       */
     virtual void OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patch);
 
-    virtual void SetSecurityMode(const PString & v)
-    { securityMode = v; }
-
-    virtual PString GetSecurityMode() const 
-    { return securityMode; }
-
-    virtual void * GetSecurityData();         
-    virtual void SetSecurityData(void *data); 
-
     void OnMediaCommand(OpalMediaCommand & command, INT extra);
 
     PDECLARE_NOTIFIER(OpalRFC2833Info, OpalRTPConnection, OnUserInputInlineRFC2833);
     PDECLARE_NOTIFIER(OpalRFC2833Info, OpalRTPConnection, OnUserInputInlineCiscoNSE);
 
+    virtual void SessionFailing(RTP_Session & session);
+
   protected:
-    PString securityMode;
-    void * securityData;
     OpalRTPSessionManager m_rtpSessions;
     OpalRFC2833Proto * rfc2833Handler;
     OpalRFC2833Proto * ciscoNSEHandler;
 
     PBoolean remoteIsNAT;
     PBoolean useRTPAggregation;
+
+#ifdef OPAL_ZRTP
+    bool zrtpEnabled;
+    PMutex zrtpConnInfoMutex;
+    OpalZRTPConnectionInfo * zrtpConnInfo;
+#endif
 };
 
 
