@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 21283 $
+ * $Revision: 21492 $
  * $Author: rjongbloed $
- * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
+ * $Date: 2008-11-10 05:41:46 +0000 (Mon, 10 Nov 2008) $
  */
 
 #ifndef OPAL_OPAL_TRANSPORT_H
@@ -93,6 +93,10 @@ class OpalTransportAddress : public PString
       const OpalTransportAddress & address
     ) const;
 
+    /**Return the underlying protocol for the transport address.
+      */
+    PCaselessString GetProto() const { return Left(Find('$')); }
+
     /**Extract the ip address from transport address.
        Returns PFalse, if the address is not an IP transport address.
       */
