@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21406 $
+ * $Author: csoutheren $
+ * $Date: 2008-10-22 11:59:46 +0000 (Wed, 22 Oct 2008) $
  */
 
 #ifndef OPAL_RTP_JITTER_H
@@ -127,7 +127,7 @@ class OpalJitterBuffer : public PObject
 
     /**Start jitter thread
       */
-    virtual void Resume(PHandleAggregator * aggregator = NULL);
+    virtual void Resume();
 
     PDECLARE_NOTIFIER(PThread, OpalJitterBuffer, JitterThreadMain);
 
