@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21860 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-23 02:24:22 +0000 (Tue, 23 Dec 2008) $
  */
 
 #ifndef OPAL_SIP_SDP_H
@@ -51,39 +51,16 @@ class SDPMediaFormat : public PObject
 {
   PCLASSINFO(SDPMediaFormat, PObject);
   public:
-    // the following values are mandated by RFC 2833
-    enum NTEEvent {
-      Digit0 = 0,
-      Digit1 = 1,
-      Digit2 = 2,
-      Digit3 = 3,
-      Digit4 = 4,
-      Digit5 = 5,
-      Digit6 = 6,
-      Digit7 = 7,
-      Digit8 = 8,
-      Digit9 = 9,
-      Star   = 10,
-      Hash   = 11,
-      A      = 12,
-      B      = 13,
-      C      = 14,
-      D      = 15,
-      Flash  = 16
-    };
-    
     SDPMediaFormat(
       RTP_DataFrame::PayloadTypes payloadType,
       const char * name = NULL
     );
 
     SDPMediaFormat(
-      const OpalMediaFormat & mediaFormat,
-      RTP_DataFrame::PayloadTypes pt,
-      const char * nteString = NULL
+      const OpalMediaFormat & mediaFormat
     );
 
-    void PrintOn(ostream & str) const;
+    virtual void PrintOn(ostream & str) const;
 
     RTP_DataFrame::PayloadTypes GetPayloadType() const { return payloadType; }
 
@@ -105,17 +82,7 @@ class SDPMediaFormat : public PObject
     bool ToNormalisedOptions();
 
   protected:
-    void AddNTEString(const PString & str);
-    void AddNTEToken(const PString & ostr);
-    PString GetNTEString() const;
-
-    void AddNSEString(const PString & str);
-    void AddNSEToken(const PString & ostr);
-    PString GetNSEString() const;
-
-    void AddNXEString(POrdinalSet & nxeSet, const PString & str);
-    void AddNXEToken(POrdinalSet & nxeSet, const PString & ostr);
-    PString GetNXEString(POrdinalSet & nxeSet) const;
+    virtual void UpdateMediaFormat() const;
 
     mutable OpalMediaFormat mediaFormat;
     RTP_DataFrame::PayloadTypes payloadType;
@@ -124,9 +91,6 @@ class SDPMediaFormat : public PObject
     PString encodingName;
     PString parameters;
     PString fmtp;
-
-    mutable POrdinalSet nteSet;     // used for NTE formats only
-    mutable POrdinalSet nseSet;     // used for NSE formats only
 };
 
 PLIST(SDPMediaFormatList, SDPMediaFormat);
@@ -171,8 +135,6 @@ class SDPMediaDescription : public PObject
     virtual bool Decode(char key, const PString & value);
     virtual bool PostDecode();
 
-    //virtual MediaType GetMediaType() const { return mediaType; }
-
     // return the string used within SDP to identify this media type
     virtual PString GetSDPMediaType() const = 0;
 
@@ -206,14 +168,16 @@ class SDPMediaDescription : public PObject
 
     virtual void RemoveSDPMediaFormat(const SDPMediaFormat & sdpMediaFormat);
 
+    virtual void CreateSDPMediaFormats(const PStringArray & tokens);
     virtual SDPMediaFormat * CreateSDPMediaFormat(const PString & portString) = 0;
 
     virtual PString GetSDPPortList() const = 0;
 
     virtual void ProcessMediaOptions(SDPMediaFormat & sdpFormat, const OpalMediaFormat & mediaFormat);
 
-  protected:
     virtual bool PrintOn(ostream & strm, const PString & str) const;
+
+  protected:
     virtual SDPMediaFormat * FindFormat(PString & str) const;
     virtual void SetPacketTime(const PString & optionName, const PString & value);
 
