@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21851 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-22 01:35:12 +0000 (Mon, 22 Dec 2008) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -43,6 +43,7 @@
 #include <opal/buildopts.h>
 #include <opal/rtpconn.h>
 #include <sip/sippdu.h>
+#include <sip/handlers.h>
 #if OPAL_VIDEO
 #include <opal/pcss.h>                  // for OpalPCSSConnection
 #include <codec/vidcodec.h>             // for OpalVideoUpdatePicture command
@@ -51,6 +52,9 @@
 class OpalCall;
 class SIPEndPoint;
 
+#define SIP_HEADER_PREFIX   "SIP-Header:"
+#define SIP_HEADER_REPLACES SIP_HEADER_PREFIX"Replaces"
+
 
 /////////////////////////////////////////////////////////////////////////
 
@@ -381,17 +385,13 @@ class SIPConnection : public OpalRTPConnection
     );
   //@}
 
-    unsigned GetNextCSeq() { return ++lastSentCSeq; }
-
-    OpalTransportAddress GetLocalAddress(WORD port = 0) const;
+    OpalTransportAddress GetDefaultSDPConnectAddress(WORD port = 0) const;
 
     OpalTransport & GetTransport() const { return *transport; }
 
     SIPEndPoint & GetEndPoint() const { return endpoint; }
-    const SIPURL & GetRequestURI() const { return m_requestURI; }
-    const PString & GetDialogFrom() const { return m_dialogFrom; }
-    const PString & GetDialogTo() const { return m_dialogTo; }
-    const PStringList & GetRouteSet() const { return routeSet; }
+    SIPDialogContext & GetDialog() { return m_dialog; }
+    const SIPDialogContext & GetDialog() const { return m_dialog; }
     SIPAuthentication * GetAuthenticator() const { return authentication; }
 
 #if OPAL_VIDEO
@@ -405,11 +405,15 @@ class SIPConnection : public OpalRTPConnection
 
     virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
 
+    virtual void OnStartTransaction(SIPTransaction & transaction);
+
+    virtual void OnReceivedMESSAGE(SIP_PDU & pdu);
+    virtual void OnMessageReceived(const SIPURL & from, const SIP_PDU & pdu);
+
   protected:
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseRetry);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnAckTimeout);
 
-    void AdjustOutgoingINVITE();
     virtual RTP_UDP *OnUseRTPSession(
       const unsigned rtpSessionId,
       const OpalMediaType & mediaType,
@@ -443,11 +447,21 @@ class SIPConnection : public OpalRTPConnection
     );
     friend class SIPInvite;
     static PBoolean WriteINVITE(OpalTransport & transport, void * param);
+    bool WriteINVITE(OpalTransport & transport);
 
     OpalTransport * CreateTransport(const OpalTransportAddress & address, PBoolean isLocalAddress = PFalse);
 
-    void UpdateRemoteAddresses(const PString & addr);
+    void UpdateRemoteAddresses();
 
+    void NotifyDialogState(
+      SIPDialogNotification::States state,
+      SIPDialogNotification::Events eventType = SIPDialogNotification::NoEvent,
+      unsigned eventCode = 0
+    );
+    void ExtractAppearanceCode(SIP_PDU & pdu);
+
+
+    // Member variables
     SIPEndPoint         & endpoint;
     OpalTransport       * transport;
     bool                  deleteTransport;
@@ -468,10 +482,9 @@ class SIPConnection : public OpalRTPConnection
     PTime                 originalInviteTime;
 
     bool                  needReINVITE;
-    PStringList           routeSet;
-    SIPURL                m_requestURI;
-    PString               m_dialogFrom;
-    PString               m_dialogTo;
+    SIPDialogContext      m_dialog;
+    PGloballyUniqueID     m_dialogNotifyId;
+    int                   m_appearanceCode;
     SIPAuthentication   * authentication;
 
     std::map<SIP_PDU::Methods, unsigned> m_lastRxCSeq;
@@ -482,7 +495,6 @@ class SIPConnection : public OpalRTPConnection
     bool                      ackReceived;
     PSafePtr<SIPTransaction>  referTransaction;
     PSafeList<SIPTransaction> forkedInvitations; // Not for re-INVITE
-    PAtomicInteger            lastSentCSeq;
 
     enum {
       ReleaseWithBYE,
@@ -493,6 +505,8 @@ class SIPConnection : public OpalRTPConnection
 
     OpalMediaFormatList remoteFormatList;
 
+    PString GetLocalPartyURL() const;
+
   protected:
     PTimer sessionTimer;
   public:
@@ -555,7 +569,7 @@ class SIP_RTP_Session : public RTP_UserData
 #endif
   //@}
 
-    void OnClearCall(const RTP_Session & /*session*/);
+    virtual void SessionFailing(RTP_Session & /*session*/);
 
   protected:
     const SIPConnection & connection; /// Owner of the RTP session
