@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21288 $
- * $Author: dsandras $
- * $Date: 2008-10-12 14:50:25 +0000 (Sun, 12 Oct 2008) $
+ * $Revision: 21807 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-16 03:28:12 +0000 (Tue, 16 Dec 2008) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -50,6 +50,9 @@
 #include <sip/sippdu.h>
 #include <sip/handlers.h> 
 
+#if OPAL_HAS_SIPIM
+#include <im/sipim.h>
+#endif
 
 class SIPRegisterHandler;
 
@@ -206,8 +209,8 @@ class SIPEndPoint : public OpalRTPEndPoint
        address is interpreted as the remote address to which the transport should connect
       */
     OpalTransport * CreateTransport(
-      const OpalTransportAddress & remoteAddress,
-      const OpalTransportAddress & localAddress = OpalTransportAddress()
+      const SIPURL & remoteURL,
+      const PString & localInterface = PString::Empty()
     );
 
     virtual void HandlePDU(
@@ -333,9 +336,8 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     /**Callback for SIP message received
      */
-    virtual void OnMessageReceived (const SIPURL & from,
-                    const PString & body);
-
+    virtual void OnMessageReceived (const SIPURL & from, const PString & body);
+    virtual void OnMessageReceived (const SIPURL & from, const SIP_PDU & pdu);
 
     /**Register to a registrar. This function is asynchronous to permit
        several registrations to occur at the same time. It can be
@@ -351,7 +353,8 @@ class SIPEndPoint : public OpalRTPEndPoint
        occur with the "best guess" of authentication parameters.
      */
     bool Register(
-      const SIPRegister::Params & params /// Registration paarameters
+      const SIPRegister::Params & params, ///< Registration paarameters
+      PString & aor                       ///< Resultant address-of-record for unregister
     );
 
     /// Registration function for backward compatibility.
@@ -377,14 +380,26 @@ class SIPEndPoint : public OpalRTPEndPoint
     bool UnregisterAll();
 
     /**Returns PTrue if the given URL has been registered 
-     * (e.g.: 6001@seconix.com).
+     * (e.g.: 6001@seconix.com). The includeOffline parameter indicates
+       if the caller is interested in if we are, to the best of our knowlegde,
+       currently registered (have had recent confirmation) or we are not sure
+       if we are registered or not, but are continually re-trying.
      */
-    PBoolean IsRegistered(const PString & aor);
+    PBoolean IsRegistered(
+      const PString & aor,          ///< Address of record to check
+      bool includeOffline = false   ///< Include offline registrations
+    );
 
     /** Returns the number of registered accounts.
      */
     unsigned GetRegistrationsCount() const { return activeSIPHandlers.GetCount(SIP_PDU::Method_REGISTER); }
 
+    /** Returns a list of registered accounts.
+     */
+    PStringList GetRegistrations(
+      bool includeOffline = false   ///< Include offline registrations
+    ) const { return activeSIPHandlers.GetAddresses(includeOffline, SIP_PDU::Method_REGISTER); }
+
     /** Information provided on the registration status. */
     struct RegistrationStatus {
       PString              m_addressofRecord;   ///< Address of record for registration
@@ -430,44 +445,58 @@ class SIPEndPoint : public OpalRTPEndPoint
      * several subscriptions to occur at the same time.
      */
     bool Subscribe(
-      SIPSubscribe::PredefinedPackages eventPackage,
-      unsigned expire,
-      const PString & to
+      SIPSubscribe::PredefinedPackages eventPackage, ///< Event package being unsubscribed
+      unsigned expire,                               ///< Expiry time in seconds
+      const PString & aor                            ///< Address-of-record for subscription
     );
     bool Subscribe(
-      const SIPSubscribe::Params & params /// Registration paarameters
+      const SIPSubscribe::Params & params, ///< Subscription paarameters
+      PString & aor                        ///< Resultant address-of-record for unsubscribe
     );
 
 
     bool Unsubscribe(
-      SIPSubscribe::PredefinedPackages eventPackage,
-      const PString & to
+      SIPSubscribe::PredefinedPackages eventPackage,  ///< Event package being unsubscribed
+      const PString & aor                             ///< Address-of-record for subscription
     );
     bool Unsubscribe(
-      const PString & eventPackage,
-      const PString & to
+      const PString & eventPackage,  ///< Event package being unsubscribed
+      const PString & aor            ///< Address-of-record for subscription
     );
 
     /**Unsubscribe all current subscriptions.
       */
     bool UnsubcribeAll(
-      SIPSubscribe::PredefinedPackages eventPackage
+      SIPSubscribe::PredefinedPackages eventPackage  ///< Event package being unsubscribed
     );
     bool UnsubcribeAll(
-      const PString & eventPackage
+      const PString & eventPackage  ///< Event package being unsubscribed
     );
 
-    /**Returns PTrue if the endpoint is subscribed to some
-     * event for the given to address.
+    /**Returns true if the endpoint is subscribed to some
+       event for the given to address. The includeOffline parameter indicates
+       if the caller is interested in if we are, to the best of our knowlegde,
+       currently subscribed (have had recent confirmation) or we are not sure
+       if we are subscribed or not, but are continually re-trying.
      */
     PBoolean IsSubscribed(
-      const PString & eventPackage,
-      const PString & to
+      const PString & eventPackage,  ///< Event package being unsubscribed
+      const PString & aor,           ///< Address-of-record for subscription
+      bool includeOffline = false    ///< Include offline subscriptions
     );
 
     /** Returns the number of registered accounts.
      */
-    unsigned GetSubscriptionCount(const PString & eventPackage) { return activeSIPHandlers.GetCount(SIP_PDU::Method_SUBSCRIBE, eventPackage); }
+    unsigned GetSubscriptionCount(
+      const SIPSubscribe::EventPackage & eventPackage  ///< Event package of subscription
+    ) { return activeSIPHandlers.GetCount(SIP_PDU::Method_SUBSCRIBE, eventPackage); }
+
+    /** Returns a list of subscribed accounts for package.
+     */
+    PStringList GetSubscriptions(
+      const SIPSubscribe::EventPackage & eventPackage, ///< Event package of subscription
+      bool includeOffline = false   ///< Include offline subscriptions
+    ) const { return activeSIPHandlers.GetAddresses(includeOffline, SIP_PDU::Method_REGISTER, eventPackage); }
 
     /**Callback called when a subscription to a SIP UA status changes.
      */
@@ -479,24 +508,96 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU::StatusCodes reason   ///< Status of subscription
     );
 
+    /** Indicate notifications for the specified event package are supported.
+      */
+    virtual bool CanNotify(
+      const PString & eventPackage ///< Event package we support
+    );
+
+    /** Send notification to all remotes that are subcribed to the event package.
+      */
+    bool Notify(
+      const SIPURL & targetAddress, ///< Address that was subscribed
+      const PString & eventPackage, ///< Event package for notification
+      const PString & body          ///< Body of notification
+    );
+
+
+    /**Callback called when dialog NOTIFY message is received
+     */
+    virtual void OnDialogInfoReceived(
+      const SIPDialogNotification & info  ///< Information on dialog state change
+    );
+
+    void SendNotifyDialogInfo(
+      const SIPDialogNotification & info  ///< Information on dialog state change
+    );
+
 
     /**Send a message to the given URL.
      */
-    PBoolean Message (
+    PBoolean Message(
       const PString & to, 
       const PString & body
     );
+    PBoolean Message(
+      const PString & to, 
+      const PString & body,
+      const PString & remoteContact, 
+      const PString & callID
+    );
     
+    /**Callback called when a message sent by the endpoint didn't reach
+     * its destination or when the proxy or remote endpoint returns
+     * an error code.
+     */
+    virtual void OnMessageFailed(
+      const SIPURL & messageUrl,
+      SIP_PDU::StatusCodes reason
+    );
+
+
+    /**Publish new state information.
+       A expire time of zero will cease to automatically update the publish.
+     */
+    bool Publish(
+      const SIPSubscribe::Params & params,
+      const PString & body,
+      PString & aor
+    );
+    bool Publish(
+      const PString & to,   ///< Address to send PUBLISH
+      const PString & body, ///< Body of PUBLISH
+      unsigned expire = 300 ///< Time between automatic updates in seconds.
+    );
+
+    /** Returns a list of published entities.
+     */
+    PStringList GetPublications(
+      const SIPSubscribe::EventPackage & eventPackage, ///< Event package for publication
+      bool includeOffline = false   ///< Include offline publications
+    ) const { return activeSIPHandlers.GetAddresses(includeOffline, SIP_PDU::Method_PUBLISH, eventPackage); }
+
 
     /**Publish new state information.
      * Only the basic & note fields of the PIDF+xml are supported for now.
      */
-    PBoolean Publish(
-      const PString & to,
-      const PString & body,
-      unsigned expire = 0
+    bool PublishPresence(
+      const SIPPresenceInfo & info,  ///< Presence info to publish
+      unsigned expire = 300          ///< Refresh time
     );
     
+    /**Callback called when presence is received
+     */
+    virtual void OnPresenceInfoReceived (
+      const SIPPresenceInfo & info  ///< Presence info publicised
+    );
+    virtual void OnPresenceInfoReceived (
+      const PString & identity,
+      const PString & basic,
+      const PString & note
+    );
+
 
     /**Send a SIP PING to the remote host
       */
@@ -504,22 +605,6 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PString & to
     );
 
-    /**Callback called when presence is received
-     */
-    virtual void OnPresenceInfoReceived (
-      const PString & user,
-      const PString & basic,
-      const PString & note
-    );
-
-    /**Callback called when a message sent by the endpoint didn't reach
-     * its destination or when the proxy or remote endpoint returns
-     * an error code.
-     */
-    virtual void OnMessageFailed(
-      const SIPURL & messageUrl,
-      SIP_PDU::StatusCodes reason);
-    
 
     void SetMIMEForm(PBoolean v) { mimeForm = v; }
     PBoolean GetMIMEForm() const { return mimeForm; }
@@ -580,7 +665,6 @@ class SIPEndPoint : public OpalRTPEndPoint
      */
     unsigned GetNextCSeq() { return ++lastSentCSeq; }
 
-    
     /**Return the SIPAuthentication for a specific realm.
      */
     PBoolean GetAuthentication(const PString & authRealm, PString & realm, PString & user, PString & password); 
@@ -590,12 +674,12 @@ class SIPEndPoint : public OpalRTPEndPoint
      * That URL can be used in the FORM field of the PDU's. 
      * The host part can be different from the registration domain.
      */
-    virtual SIPURL GetRegisteredPartyName(const SIPURL &);
+    virtual SIPURL GetRegisteredPartyName(const SIPURL & remoteURL, const OpalTransport & transport);
 
 
     /**Return the default registered party name URL.
      */
-    virtual SIPURL GetDefaultRegisteredPartyName();
+    virtual SIPURL GetDefaultRegisteredPartyName(const OpalTransport & transport);
     
 
     /**Return the contact URL for the given host and user name
@@ -646,6 +730,14 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     
+    /**Get the default line appearance code for new connections.
+      */
+    int GetDefaultAppearanceCode() const { return m_defaultAppearanceCode; }
+
+    /**Set the default line appearance code for new connections.
+      */
+    void SetDefaultAppearanceCode(int code) { m_defaultAppearanceCode = code; }
+
     /**Get the User Agent for this endpoint.
        Default behaviour returns an empty string so the SIPConnection builds
        a valid string from the productInfo data.
@@ -663,11 +755,6 @@ class SIPEndPoint : public OpalRTPEndPoint
       */
     virtual unsigned GetAllowedMethods() const;
 
-    PBoolean SendResponse(
-      SIP_PDU::StatusCodes code, 
-      OpalTransport & transport, 
-      SIP_PDU & pdu
-    );
 
     /**NAT Binding Refresh Method
      */
@@ -685,6 +772,12 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     virtual SIPRegisterHandler * CreateRegisterHandler(const SIPRegister::Params & params);
 
+    virtual void OnStartTransaction(SIPConnection & conn, SIPTransaction & transaction);
+
+#if OPAL_HAS_SIPIM
+    virtual OpalSIPIMManager & GetSIPIMManager() { return m_sipIMManager; }
+#endif
+
   protected:
     PDECLARE_NOTIFIER(PThread, SIPEndPoint, TransportThreadMain);
     PDECLARE_NOTIFIER(PTimer, SIPEndPoint, NATBindingRefresh);
@@ -703,15 +796,17 @@ class SIPEndPoint : public OpalRTPEndPoint
     PTimeInterval registrarTimeToLive;
     PTimeInterval notifierTimeToLive;
     PTimeInterval natBindingTimeout;
-    
+
+    bool              m_shuttingDown;
     SIPHandlersList   activeSIPHandlers;
 
     PSafeDictionary<PString, SIPTransaction> transactions;
 
     PTimer                  natBindingTimer;
     NATBindingRefreshMethod natMethod;
-    
-    PAtomicInteger          lastSentCSeq;    
+    PAtomicInteger          lastSentCSeq;
+    unsigned                m_dialogNotifyVersion;
+    int                     m_defaultAppearanceCode;
 
     struct SIP_PDU_Work
     {
@@ -765,6 +860,10 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     friend void InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry & entry);
     friend void InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
+
+#if OPAL_HAS_SIPIM
+    OpalSIPIMManager m_sipIMManager;
+#endif
 };
 
 
