@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20479 $
+ * $Revision: 21283 $
  * $Author: rjongbloed $
- * $Date: 2008-06-21 07:42:09 +0000 (Sat, 21 Jun 2008) $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
 #ifndef _Jester_MAIN_H
@@ -56,6 +56,9 @@
 #include <ptclib/delaychan.h>
 #include <ptclib/random.h>
 
+#if !OPAL_IAX2
+#error Cannot compile without IAX2
+#endif
 
 
 /////////////////////////////////////////////////////////////////////////////
