@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21618 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-25 00:49:58 +0000 (Tue, 25 Nov 2008) $
  */
 
 #ifndef _OpenPhone_VERSION_H
@@ -38,10 +38,10 @@
 #define	COPYRIGHT_HOLDER	"Vox Lucida Pty. Ltd. (Australia)"
 
 
-#define MAJOR_VERSION 2
-#define MINOR_VERSION 4
-#define BUILD_TYPE    ReleaseCode
-#define BUILD_NUMBER 1
+#define MAJOR_VERSION 3
+#define MINOR_VERSION 5
+#define BUILD_TYPE    BetaCode
+#define BUILD_NUMBER 0
 
 
 #endif  // _OpenPhone_VERSION_H
