@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21857 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-22 04:03:10 +0000 (Mon, 22 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -47,8 +47,6 @@
 #include <t38/t38proto.h>
 #endif
 
-#include <t38/t38proto.h>
-
 #include <opal/transcoders.h>
 #include <lids/lidep.h>
 #include <ptclib/pstun.h>
@@ -171,9 +169,9 @@ void SimpleOpalProcess::Main()
 
 
   if (args.HasOption('h') || (!args.HasOption('l') && args.GetCount() == 0)) {
-    cout << "Usage : " << GetName() << " [options] -l\n"
-            "      : " << GetName() << " [options] [alias@]hostname   (no gatekeeper)\n"
-            "      : " << GetName() << " [options] alias[@hostname]   (with gatekeeper)\n"
+    cout << "Usage : " << GetFile().GetTitle() << " [options] -l\n"
+            "      : " << GetFile().GetTitle() << " [options] [alias@]hostname   (no gatekeeper)\n"
+            "      : " << GetFile().GetTitle() << " [options] alias[@hostname]   (with gatekeeper)\n"
             "General options:\n"
             "  -l --listen             : Listen for incoming calls.\n"
             "  -d --dial-peer spec     : Set dial peer for routing calls (see below)\n"
@@ -390,7 +388,6 @@ MyManager::MyManager()
 #endif
 #if OPAL_FAX
   faxEP = NULL;
-  t38EP = NULL;
 #endif
 
   pauseBeforeDialing = PFalse;
@@ -412,13 +409,13 @@ PBoolean MyManager::Initialise(PArgList & args)
 #if OPAL_VIDEO
   // Set the various global options
   if (args.HasOption("rx-video"))
-    autoStartReceiveVideo = PTrue;
+    SetAutoStartReceiveVideo(true);
   if (args.HasOption("no-rx-video"))
-    autoStartReceiveVideo = PFalse;
+    SetAutoStartReceiveVideo(false);
   if (args.HasOption("tx-video"))
-    autoStartTransmitVideo = PTrue;
+    SetAutoStartTransmitVideo(true);
   if (args.HasOption("no-tx-video"))
-    autoStartTransmitVideo = PFalse;
+    SetAutoStartTransmitVideo(false);
 
   if (args.HasOption("grabber")) {
     PVideoDevice::OpenArgs video = GetVideoInputDevice();
@@ -700,9 +697,6 @@ PBoolean MyManager::Initialise(PArgList & args)
   {
     OpalMediaFormat fmt(OpalT38); // Force instantiation of T.38 media format
     faxEP = new OpalFaxEndPoint(*this);
-    t38EP = new OpalT38EndPoint(*this);
-
-    allMediaFormats += t38EP->GetMediaFormats();
     allMediaFormats += faxEP->GetMediaFormats();
   }
 #endif
@@ -790,8 +784,9 @@ PBoolean MyManager::Initialise(PArgList & args)
 #endif
 
 #if OPAL_FAX
-  if (t38EP != NULL) {
-      AddRouteEntry("sip:.*  = t38:<da>");
+  if (faxEP != NULL) {
+    AddRouteEntry("sip:.*  = t38:<da>");
+    AddRouteEntry("sip:.*  = fax:<da>");
   }
 #endif
 
