@@ -24,15 +24,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21464 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-06 07:27:22 +0000 (Thu, 06 Nov 2008) $
  */
 
 #include <ptlib.h>
 #include <opal/buildopts.h>
 
 #include <opal/mediafmt.h>
+#include <codec/opalplugin.h>
 
 
 #define new PNEW
@@ -82,7 +83,7 @@ const OpalAudioFormat & GetOpalGSMAMR()
         }
 #endif
 
-        AddOption(new OpalMediaOptionString("Media Packetization", true, "RFC3267"));
+        AddOption(new OpalMediaOptionString(PLUGINCODEC_MEDIA_PACKETIZATIONS, true, "RFC3267,RFC4867"));
       }
   } const GSMAMR;
   return GSMAMR;
