@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21846 $
+ * $Author: ykiryanov $
+ * $Date: 2008-12-19 06:45:12 +0000 (Fri, 19 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -189,7 +189,7 @@ static PBoolean WriteGRQ(H323Transport & transport, void * param)
     return false;
 
   // Check if interface is used by signalling channel listeners
-  OpalTransportAddressArray interfaces = endpoint.GetInterfaceAddresses();
+  OpalTransportAddressArray interfaces = endpoint.GetInterfaceAddresses(true, &transport);
   bool notInterface = true;
   for (PINDEX i = 0; i < interfaces.GetSize(); ++i) {
     PIPSocket::Address ifAddress;
@@ -422,6 +422,7 @@ PBoolean H323Gatekeeper::RegistrationRequest(PBoolean autoReg, PBoolean didGkDis
   }
 
   for (PINDEX i = 0; i < listeners.GetSize(); i++)
+   if(listeners[i].GetProto() == "tcp")
     listeners[i].SetPDU(rrq.m_callSignalAddress, *transport);
 
   endpoint.SetEndpointTypeInfo(rrq.m_terminalType);
@@ -662,6 +663,13 @@ PBoolean H323Gatekeeper::OnReceiveRegistrationReject(const H225_RegistrationReje
     SetAlternates(rrj.m_altGKInfo.m_alternateGatekeeper,
                   rrj.m_altGKInfo.m_altGKisPermanent);
 
+  // Update registration fail reason from last request fail reason
+  switch(lastRequest->rejectReason) {
+	case H225_RegistrationRejectReason::e_duplicateAlias : registrationFailReason = DuplicateAlias; break;
+	case H225_RegistrationRejectReason::e_securityDenial: registrationFailReason = SecurityDenied; break;
+	default: ;
+  }
+  
   endpoint.OnRegistrationReject();
 
   return PTrue;
@@ -1284,7 +1292,7 @@ PBoolean H323Gatekeeper::DisengageRequest(const H323Connection & connection, uns
 
   drq.IncludeOptionalField(H225_DisengageRequest::e_terminationCause);
   drq.m_terminationCause.SetTag(H225_CallTerminationCause::e_releaseCompleteReason);
-  Q931::CauseValues cause = H323TranslateFromCallEndReason(connection, drq.m_terminationCause);
+  Q931::CauseValues cause = H323TranslateFromCallEndReason(connection.GetCallEndReason(), drq.m_terminationCause);
   if (cause != Q931::ErrorInCauseIE) {
     drq.m_terminationCause.SetTag(H225_CallTerminationCause::e_releaseCompleteCauseIE);
     PASN_OctetString & rcReason = drq.m_terminationCause;
