@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20836 $
+ * $Revision: 21543 $
  * $Author: rjongbloed $
- * $Date: 2008-09-03 05:24:34 +0000 (Wed, 03 Sep 2008) $
+ * $Date: 2008-11-18 03:30:26 +0000 (Tue, 18 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -730,7 +730,7 @@ H323Capability::MainTypes H323AudioCapability::GetMainType() const
 
 unsigned H323AudioCapability::GetDefaultSessionID() const
 {
-  return OpalMediaFormat::DefaultAudioSessionID;
+  return DefaultAudioSessionID;
 }
 
 
@@ -1126,7 +1126,7 @@ PBoolean H323VideoCapability::OnReceivedPDU(const H245_VideoCapability & pdu, Co
 
 unsigned H323VideoCapability::GetDefaultSessionID() const
 {
-  return OpalMediaFormat::DefaultVideoSessionID;
+  return DefaultVideoSessionID;
 }
 
 
@@ -1876,20 +1876,14 @@ H323Capabilities::H323Capabilities()
 H323Capabilities::H323Capabilities(const H323Connection & connection,
                                    const H245_TerminalCapabilitySet & pdu)
 {
-  H323Capabilities allCapabilities;
-  const H323Capabilities & localCapabilities = connection.GetLocalCapabilities();
-  for (PINDEX c = 0; c < localCapabilities.GetSize(); c++)
-    allCapabilities.Add(allCapabilities.Copy(localCapabilities[c]));
-  allCapabilities.AddAllCapabilities(connection.GetEndPoint(), 0, 0, "*");
-  H323_UserInputCapability::AddAllCapabilities(allCapabilities, P_MAX_INDEX, P_MAX_INDEX);
+  PTRACE(4, "H323\tH323Capabilities(ctor)");
 
   /** If mediaPacketization information is available, use this with the FindCapability() logic.
    *  Certain codecs, such as H.263, need additional information in order to match the specific
    *  version of the codec against possibly multiple codec with the same 'subtype' such as
    *  e_h263VideoCapability.
    */
-  PTRACE(4, "H323\tH323Capabilities(ctor) build list of mediaPacketization if available.");
-
+  mediaPacketizations += "RFC2190";  // Always supported
   const H245_MultiplexCapability * muxCap = NULL;
   if (pdu.HasOptionalField(H245_TerminalCapabilitySet::e_multiplexCapability)) {
     muxCap = &pdu.m_multiplexCapability;
@@ -1904,8 +1898,8 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
         for (PINDEX i = 0; i < mediaPacket.m_rtpPayloadType.GetSize(); i++) {
           PString mediaPacketization = H323GetRTPPacketization(mediaPacket.m_rtpPayloadType[i]);
           if (!mediaPacketization.IsEmpty()) {
-              mediaPacketizations += mediaPacketization;
-              PTRACE(4, "H323\tH323Capabilities(ctor) Appended mediaPacketization=" << mediaPacketization << ", mediaPacketization count=" << mediaPacketizations.GetSize());
+            mediaPacketizations += mediaPacketization;
+            PTRACE(4, "H323\tH323Capabilities(ctor) Appended mediaPacketization=" << mediaPacketization << ", mediaPacketization count=" << mediaPacketizations.GetSize());
           }
         } // for ... m_rtpPayloadType.GetSize()
       } // e_rtpPayloadType
@@ -1914,12 +1908,47 @@ H323Capabilities::H323Capabilities(const H323Connection & connection,
 
   // Decode out of the PDU, the list of known codecs.
   if (pdu.HasOptionalField(H245_TerminalCapabilitySet::e_capabilityTable)) {
+    H323Capabilities allCapabilities;
+    const H323Capabilities & localCapabilities = connection.GetLocalCapabilities();
+    for (PINDEX c = 0; c < localCapabilities.GetSize(); c++)
+      allCapabilities.Add(allCapabilities.Copy(localCapabilities[c]));
+    allCapabilities.AddAllCapabilities(connection.GetEndPoint(), 0, 0, "*");
+    H323_UserInputCapability::AddAllCapabilities(allCapabilities, P_MAX_INDEX, P_MAX_INDEX);
+    allCapabilities.SetMediaPacketizations(mediaPacketizations);
+
     for (PINDEX i = 0; i < pdu.m_capabilityTable.GetSize(); i++) {
       if (pdu.m_capabilityTable[i].HasOptionalField(H245_CapabilityTableEntry::e_capability)) {
         H323Capability * capability = allCapabilities.FindCapability(pdu.m_capabilityTable[i].m_capability);
         if (capability != NULL) {
           H323Capability * copy = (H323Capability *)capability->Clone();
           copy->SetCapabilityNumber(pdu.m_capabilityTable[i].m_capabilityTableEntryNumber);
+          if (mediaPacketizations.GetSize() != 0) { // also update the mediaPacketizations option
+            OpalMediaFormat & mediaFormat = copy->GetWritableMediaFormat();
+            PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                          mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+            if (!packetizationString.IsEmpty()) {
+              // remove packetizations not signaled by the remote party
+              PStringArray packetizations = packetizationString.Tokenise(",");
+              for (PINDEX j = packetizations.GetSize(); j > 0; j--) {
+                if (!mediaPacketizations.Contains(packetizations[j-1])) {
+                  packetizations.RemoveAt(j-1);
+                }
+              }
+
+              // construct the new packetization string
+              packetizationString.MakeEmpty();
+              if (packetizations.GetSize() == 0) { // should not happen actually
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), PString::Empty());
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  PString::Empty());
+              } else {
+                packetizationString = packetizations[0];
+                for (PINDEX j = 1; j < packetizations.GetSize(); j++)
+                  packetizationString += "," + packetizations[j];
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationsOption(), packetizationString);
+                mediaFormat.SetOptionString(OpalMediaFormat::MediaPacketizationOption(),  packetizations[0]);
+              }
+            }
+          }
           if (copy->OnReceivedPDU(pdu.m_capabilityTable[i].m_capability) &&
               copy->GetWritableMediaFormat().ToNormalisedOptions())
             table.Append(copy);
@@ -2057,9 +2086,14 @@ PINDEX H323Capabilities::AddMediaFormat(PINDEX descriptorNum,
     if (capability != NULL) {
       capability->GetWritableMediaFormat() = mediaFormat;
       reply = SetCapability(descriptorNum, simultaneous, capability);
-      PString packetization = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption());
-      if (!packetization.IsEmpty())
-        mediaPacketizations += packetization;
+      PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                    mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+      if (!packetizationString.IsEmpty()) {
+        PStringArray packetizations = packetizationString.Tokenise(",");
+        for (PINDEX i = 0; i < packetizations.GetSize(); i++) {
+          mediaPacketizations += packetizations[i];
+        }
+      }
     }
   }
 
@@ -2271,32 +2305,14 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     H323Capability & capability = table[i];
 
-    /** If mediaPacketization information is available, use this with the FindCapability() logic.
-     *  Certain codecs, such as H.263, need additional information in order to match the specific
-     *  version of the codec against possibly multiple codec with the same 'subtype' such as
-     *  e_h263VideoCapability.
-     */
-    if (!mediaPacketizations.IsEmpty()) {
-      PString capabilitiesPacketization = capability.GetMediaFormat().GetOptionString(OpalMediaFormat::MediaPacketizationOption());
-      if (!capabilitiesPacketization.IsEmpty()) {
-        PINDEX j;
-        for (j = 0; j < mediaPacketizations.GetSize(); j++) {
-          if (mediaPacketizations.GetKeyAt(j) == capabilitiesPacketization)
-            break;
-        }
-        if (j >= mediaPacketizations.GetSize())
-          return false;
-      }
-    }
-
+    bool found = false;
     switch (cap.GetTag()) {
       case H245_Capability::e_receiveAudioCapability :
       case H245_Capability::e_transmitAudioCapability :
       case H245_Capability::e_receiveAndTransmitAudioCapability :
         if (capability.GetMainType() == H323Capability::e_Audio) {
           const H245_AudioCapability & audio = cap;
-          if (capability.IsMatch(audio))
-            return &capability;
+          found = capability.IsMatch(audio);
         }
         break;
 
@@ -2305,8 +2321,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
       case H245_Capability::e_receiveAndTransmitVideoCapability :
         if (capability.GetMainType() == H323Capability::e_Video) {
           const H245_VideoCapability & video = cap;
-          if (capability.IsMatch(video))
-            return &capability;
+          found = capability.IsMatch(video);
         }
         break;
 
@@ -2315,8 +2330,7 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
       case H245_Capability::e_receiveAndTransmitDataApplicationCapability :
         if (capability.GetMainType() == H323Capability::e_Data) {
           const H245_DataApplicationCapability & data = cap;
-          if (capability.IsMatch(data.m_application))
-            return &capability;
+          found = capability.IsMatch(data.m_application);
         }
         break;
 
@@ -2325,14 +2339,40 @@ H323Capability * H323Capabilities::FindCapability(const H245_Capability & cap) c
       case H245_Capability::e_receiveAndTransmitUserInputCapability :
         if (capability.GetMainType() == H323Capability::e_UserInput) {
           const H245_UserInputCapability & ui = cap;
-          if (capability.IsMatch(ui))
-            return &capability;
+          found = capability.IsMatch(ui);
         }
         break;
 
       case H245_Capability::e_receiveRTPAudioTelephonyEventCapability :
         return FindCapability(H323Capability::e_UserInput, SignalToneRFC2833_SubType);
     }
+
+    if (found) {
+      if (mediaPacketizations.IsEmpty())
+        return &capability;
+
+      /** If mediaPacketization information is available, use this with the FindCapability() logic.
+       *  Certain codecs, such as H.263, need additional information in order to match the specific
+       *  version of the codec against possibly multiple codec with the same 'subtype' such as
+       *  e_h263VideoCapability.
+       */
+      OpalMediaFormat mediaFormat = capability.GetMediaFormat();
+      PString packetizationString = mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationsOption(),
+                                    mediaFormat.GetOptionString(OpalMediaFormat::MediaPacketizationOption()));
+      if (packetizationString.IsEmpty())
+        return &capability;
+
+      // require at least one of the media format's packetizations to be in the mediaPacketizations list
+      PStringArray packetizations = packetizationString.Tokenise(",");
+      for (PINDEX j = 0; j < packetizations.GetSize(); j++) {
+        for (PINDEX k = 0; k < mediaPacketizations.GetSize(); k++) {
+          if (mediaPacketizations.GetKeyAt(k) == packetizations[j])
+            return &capability;
+        }
+      }
+
+      PTRACE(4, "H323\tUnsupported media packetization " << packetizationString << ", not using capability " << cap);
+    }
   }
 
 #if PTRACING
@@ -2547,18 +2587,8 @@ void H323Capabilities::BuildPDU(const H323Connection & connection,
       capability.GetWritableMediaFormat().ToCustomisedOptions();
       capability.OnSendingPDU(entry.m_capability);
 
-      h225_0.m_mediaPacketizationCapability.m_rtpPayloadType.SetSize(rtpPacketizationCount+1);
-      if (H323SetRTPPacketization(h225_0.m_mediaPacketizationCapability.m_rtpPayloadType[rtpPacketizationCount],
-                                                    capability.GetMediaFormat(), RTP_DataFrame::IllegalPayloadType)) {
-        // Check if already in list
-        PINDEX test;
-        for (test = 0; test < rtpPacketizationCount; test++) {
-          if (h225_0.m_mediaPacketizationCapability.m_rtpPayloadType[test] == h225_0.m_mediaPacketizationCapability.m_rtpPayloadType[rtpPacketizationCount])
-            break;
-        }
-        if (test == rtpPacketizationCount)
-          rtpPacketizationCount++;
-      }
+      H323SetRTPPacketization(h225_0.m_mediaPacketizationCapability.m_rtpPayloadType, rtpPacketizationCount,
+                              capability.GetMediaFormat(), RTP_DataFrame::IllegalPayloadType);
     }
   }
 
