@@ -24,49 +24,31 @@
  *
  * The author of this code is Derek J Smithies
  *
- *
- *
- * $Log: ies.cxx,v $
- * Revision 1.3.2.1  2006/01/31 08:10:37  csoutheren
- * Backported from CVS head
- *
- * Revision 1.4  2006/01/31 03:28:48  csoutheren
- * Removed compile warnings and changed functions args to const refs
- *
- * Revision 1.3  2005/08/26 03:07:38  dereksmithies
- * Change naming convention, so all class names contain the string "IAX2"
- *
- * Revision 1.2  2005/08/24 01:38:38  dereksmithies
- * Add encryption, iax2 style. Numerous tidy ups. Use the label iax2, not iax
- *
- * Revision 1.1  2005/07/30 07:01:33  csoutheren
- * Added implementation of IAX2 (Inter Asterisk Exchange 2) protocol
- * Thanks to Derek Smithies of Indranet Technologies Ltd. for
- * writing and contributing this code
- *
- *
- *
+ * $Revision: 21283 $
+ * $Author: rjongbloed $
+ * $Date: 2008-10-11 07:10:58 +0000 (Sat, 11 Oct 2008) $
  */
 
-
 #include <ptlib.h>
-#include <ptclib/cypher.h>
+#include <opal/buildopts.h>
 
+#if OPAL_IAX2
 
 #ifdef P_USE_PRAGMA
 #pragma implementation "ies.h"
 #endif
 
-
 #include <iax2/ies.h>
 #include <iax2/frame.h>
 #include <iax2/causecode.h>
 
+#include <ptclib/cypher.h>
+
 #define new PNEW
 
 IAX2Ie::IAX2Ie()
 {
-  validData = FALSE;
+  validData = PFalse;
 }
 
 IAX2Ie * IAX2Ie::BuildInformationElement(BYTE _typeCode, BYTE length, BYTE *srcData)
@@ -156,7 +138,7 @@ void IAX2Ie::WriteBinary(void *_data, PINDEX &writeIndex)
 
 IAX2IeNone::IAX2IeNone(BYTE /*length*/, BYTE * /*srcData*/)
 {
-  validData = TRUE;
+  validData = PTrue;
 }
 
 
@@ -170,7 +152,7 @@ void IAX2IeNone::PrintOn(ostream & str) const
 IAX2IeDateAndTime::IAX2IeDateAndTime(BYTE length, BYTE *srcData)
 {
   if (length != sizeof(unsigned int)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
@@ -183,7 +165,7 @@ IAX2IeDateAndTime::IAX2IeDateAndTime(BYTE length, BYTE *srcData)
   int year   = ((tmp >> 25) & 0x7f) + 2000;
   dataValue = PTime(second, minute, hour, day, month, year, PTime::Local);
   
-  validData = TRUE;
+  validData = PTrue;
 }
 
 
@@ -213,11 +195,11 @@ IAX2IeByte::IAX2IeByte(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(BYTE)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   dataValue = *((BYTE *)srcData);
 }
 
@@ -235,11 +217,11 @@ IAX2IeChar::IAX2IeChar(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(BYTE)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   dataValue = *((char *)srcData);
 }
 
@@ -258,11 +240,11 @@ IAX2IeUShort::IAX2IeUShort(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(unsigned short)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   dataValue = (unsigned short)((srcData[0] << 8) | srcData[1]);
 }
 
@@ -286,11 +268,11 @@ IAX2IeShort::IAX2IeShort(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(short)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   dataValue = (short)((srcData[0] << 8) | srcData[1]);
 }
 
@@ -314,11 +296,11 @@ IAX2IeInt::IAX2IeInt(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(int)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   dataValue = (srcData[0] << 24) | (srcData[1] << 16) | (srcData[2] << 8) | srcData[3];
 }
 
@@ -345,11 +327,11 @@ IAX2IeUInt::IAX2IeUInt(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(unsigned int)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   dataValue = (srcData[0] << 24) | (srcData[1] << 16) | (srcData[2] << 8) | srcData[3];     
 }
 
@@ -374,7 +356,7 @@ void IAX2IeUInt::WriteBinary(BYTE *data)
 IAX2IeString::IAX2IeString(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
-  validData = TRUE;
+  validData = PTrue;
   dataValue = PString((const char *)srcData, length);
 }
 
@@ -404,13 +386,13 @@ BYTE IAX2IeString::GetLengthOfData()
 void IAX2IeString::SetData(const PString & newData) 
 { 
   dataValue = newData; 
-  validData = TRUE; 
+  validData = PTrue; 
 }
 
 void IAX2IeString::SetData(const char * newData) 
 { 
   dataValue = PString(newData); 
-  validData = TRUE; 
+  validData = PTrue; 
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -419,11 +401,11 @@ IAX2IeSockaddrIn::IAX2IeSockaddrIn(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
   if (length != sizeof(sockaddr_in)) {
-    validData = FALSE;
+    validData = PFalse;
     return;
   }
   
-  validData = TRUE;
+  validData = PTrue;
   
   sockaddr_in a = * (sockaddr_in *)(srcData);
   portNumber = a.sin_port;
@@ -454,7 +436,7 @@ void IAX2IeSockaddrIn::WriteBinary(BYTE *data)
 IAX2IeBlockOfData::IAX2IeBlockOfData(BYTE length, BYTE *srcData)
   : IAX2Ie()
 {
-  validData = TRUE;
+  validData = PTrue;
   
   dataValue = PBYTEArray(srcData, length);
 }
@@ -1018,6 +1000,8 @@ void IAX2IeReceivedOoo::PrintOn(ostream & str) const
 ////////////////////////////////////////////////////////////////////////////////
 
 
+#endif // OPAL_IAX2
+
 ////////////////////////////////////////////////////////////////////////////////
 /* The comment below is magic for those who use emacs to edit this file. */
 /* With the comment below, the tab key does auto indent to 4 spaces.     */
