@@ -24,14 +24,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21447 $
+ * $Author: csoutheren $
+ * $Date: 2008-11-04 07:37:55 +0000 (Tue, 04 Nov 2008) $
  */
 
 #include <opal/buildopts.h>
 
+#pragma warning(disable:4206)
+
 #if OPAL_JAVA
-#include "java_swig_wrapper.c"
+#include "java_swig_wrapper.inc"
 #endif
 
