@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20660 $
- * $Author: rjongbloed $
- * $Date: 2008-08-13 03:58:41 +0200 (mer 13 aoû 2008) $
+ * $Revision: 21028 $
+ * $Author: csoutheren $
+ * $Date: 2008-09-16 23:32:30 +0000 (Tue, 16 Sep 2008) $
  */
 
 #ifdef __GNUC__
@@ -35,6 +35,8 @@
 
 #include <ptlib.h>
 
+#include <opal/buildopts.h>
+
 #include <lids/lidpluginmgr.h>
 #include <ptclib/dtmf.h>
 
@@ -57,9 +59,13 @@ OpalPluginLIDManager::~OpalPluginLIDManager()
 void OpalPluginLIDManager::OnLoadPlugin(PDynaLink & dll, INT code)
 {
   PluginLID_GetDefinitionsFunction getDefinitions;
-  if (!dll.GetFunction(PString(signatureFunctionName), (PDynaLink::Function &)getDefinitions)) {
-    PTRACE(3, "LID Plugin\tDLL " << dll.GetName() << " is not a plugin LID");
-    return;
+  {
+    PDynaLink::Function fn;
+    if (!dll.GetFunction(PString(signatureFunctionName), fn)) {
+      PTRACE(3, "LID Plugin\tDLL " << dll.GetName() << " is not a plugin LID");
+      return;
+    }
+    getDefinitions = (PluginLID_GetDefinitionsFunction)fn;
   }
 
   unsigned count;
@@ -193,8 +199,9 @@ static ostream & operator<<(ostream & stream, PluginLID_Errors error)
 
 PluginLID_Errors OpalPluginLID::CheckError(PluginLID_Errors error, const char * fnName) const
 {
-  if (error != PluginLID_NoError && error != PluginLID_UnimplementedFunction && error != PluginLID_NoMoreNames)
+  if (error != PluginLID_NoError && error != PluginLID_UnimplementedFunction && error != PluginLID_NoMoreNames) {
     PTRACE(2, "LID Plugin\tFunction " << fnName << " in " << m_definition.name << " returned error " << error);
+  }
 
   osError = error;
   return error;
