@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21655 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-30 05:16:47 +0000 (Sun, 30 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -216,7 +216,8 @@ PBoolean OpalCall::OnConnected(OpalConnection & connection)
   UnlockReadOnly();
 
   if (ok) {
-    if (!manager.MakeConnection(*this, m_partyB, NULL, 0, connection.GetStringOptions()))
+    if (!manager.MakeConnection(*this, m_partyB, NULL, 0,
+                                const_cast<OpalConnection::StringOptions *>(&connection.GetStringOptions())))
       connection.Release(OpalConnection::EndedByNoUser);
     return OnSetUp(connection);
   }
