@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21860 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-12-23 02:24:22 +0000 (Tue, 23 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -73,8 +73,12 @@ static class InstantiateMe
 
 AUDIO_FORMAT(PCM16,          MaxPayloadType, "",     16, 8,  240,  0, 256,  8000);
 AUDIO_FORMAT(PCM16_16KHZ,    MaxPayloadType, "",     32,16,  240,  0, 256, 16000);
+AUDIO_FORMAT(PCM16_32KHZ,    MaxPayloadType, "",     64,32,  240,  0, 256, 32000);
+AUDIO_FORMAT(PCM16_48KHZ,    MaxPayloadType, "",     96,48,  240,  0, 256, 48000);
 AUDIO_FORMAT(L16_MONO_8KHZ,  L16_Mono,       "L16",  16, 8,  240, 30, 256,  8000);
 AUDIO_FORMAT(L16_MONO_16KHZ, L16_Mono,       "L16",  32,16,  240, 30, 256, 16000);
+AUDIO_FORMAT(L16_MONO_32KHZ, L16_Mono,       "L16",  64,32,  240, 30, 256, 32000);
+AUDIO_FORMAT(L16_MONO_48KHZ, L16_Mono,       "L16",  96,48,  240, 30, 256, 48000);
 AUDIO_FORMAT(G711_ULAW_64K,  PCMU,           "PCMU",  8, 8,  240, 20, 256,  8000);
 AUDIO_FORMAT(G711_ALAW_64K,  PCMA,           "PCMA",  8, 8,  240, 20, 256,  8000);
 
@@ -115,33 +119,41 @@ const OpalAudioFormat & GetOpalL16_STEREO_48KHZ()
 
 const OpalMediaFormat & GetOpalRFC2833()
 {
-  static const OpalMediaFormat RFC2833(
-    OPAL_RFC2833,
-    "userinput",
-    (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
-    "telephone-event",
-    true,   // Needs jitter
-    32*(1000/50), // bits/sec  (32 bits every 50ms)
-    4,      // bytes/frame
-    150*8,  // 150 millisecond
-    OpalMediaFormat::AudioClockRate
-  );
+  static struct OpalRFC2833MediaFormat : public OpalMediaFormat {
+    OpalRFC2833MediaFormat()
+      : OpalMediaFormat(OPAL_RFC2833,
+                        "userinput",
+                        (RTP_DataFrame::PayloadTypes)101,  // Set to this for Cisco compatibility
+                        "telephone-event",
+                        true,   // Needs jitter
+                        32*(1000/50), // bits/sec  (32 bits every 50ms)
+                        4,      // bytes/frame
+                        150*8,  // 150 millisecond
+                        OpalMediaFormat::AudioClockRate)
+    {
+      AddOption(new OpalMediaOptionString("FMTP", false, "0-16,32,36"));
+    }
+  } const RFC2833;
   return RFC2833;
 }
 
 const OpalMediaFormat & GetOpalCiscoNSE()
 {
-  static const OpalMediaFormat CiscoNSE(
-    OPAL_CISCONSE,
-    "userinput",
-    (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
-    "NSE",
-    true,   // Needs jitter
-    32*(1000/50), // bits/sec  (32 bits every 50ms)
-    4,      // bytes/frame
-    150*8,  // 150 millisecond
-    OpalMediaFormat::AudioClockRate
-  );
+  static struct OpalCiscoNSEMediaFormat : public OpalMediaFormat {
+    OpalCiscoNSEMediaFormat()
+      : OpalMediaFormat(OPAL_CISCONSE,
+                        "userinput",
+                        (RTP_DataFrame::PayloadTypes)100,  // Set to this for Cisco compatibility
+                        "NSE",
+                        true,   // Needs jitter
+                        32*(1000/50), // bits/sec  (32 bits every 50ms)
+                        4,      // bytes/frame
+                        150*8,  // 150 millisecond
+                        OpalMediaFormat::AudioClockRate)
+    {
+      AddOption(new OpalMediaOptionString("FMTP", false, "192,193"));
+    }
+  } const CiscoNSE;
   return CiscoNSE;
 }
 
@@ -613,14 +625,16 @@ void OpalMediaOptionOctets::SetValue(const BYTE * data, PINDEX length)
 
 /////////////////////////////////////////////////////////////////////////////
 
-const PString & OpalMediaFormat::NeedsJitterOption()  { static PString s = PLUGINCODEC_OPTION_NEEDS_JITTER;   return s; }
-const PString & OpalMediaFormat::MaxFrameSizeOption() { static PString s = PLUGINCODEC_OPTION_MAX_FRAME_SIZE; return s; }
-const PString & OpalMediaFormat::FrameTimeOption()    { static PString s = PLUGINCODEC_OPTION_FRAME_TIME;     return s; }
-const PString & OpalMediaFormat::ClockRateOption()    { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;     return s; }
-const PString & OpalMediaFormat::MaxBitRateOption()   { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;   return s; }
-const PString & OpalMediaFormat::TargetBitRateOption(){ static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
+const PString & OpalMediaFormat::NeedsJitterOption()   { static PString s = PLUGINCODEC_OPTION_NEEDS_JITTER;    return s; }
+const PString & OpalMediaFormat::MaxFrameSizeOption()  { static PString s = PLUGINCODEC_OPTION_MAX_FRAME_SIZE;  return s; }
+const PString & OpalMediaFormat::FrameTimeOption()     { static PString s = PLUGINCODEC_OPTION_FRAME_TIME;      return s; }
+const PString & OpalMediaFormat::ClockRateOption()     { static PString s = PLUGINCODEC_OPTION_CLOCK_RATE;      return s; }
+const PString & OpalMediaFormat::MaxBitRateOption()    { static PString s = PLUGINCODEC_OPTION_MAX_BIT_RATE;    return s; }
+const PString & OpalMediaFormat::TargetBitRateOption() { static PString s = PLUGINCODEC_OPTION_TARGET_BIT_RATE; return s; }
+
 #if OPAL_H323
-const PString & OpalMediaFormat::MediaPacketizationOption(){ static PString s = PLUGINCODEC_MEDIA_PACKETIZATION; return s; }
+const PString & OpalMediaFormat::MediaPacketizationOption()  { static PString s = PLUGINCODEC_MEDIA_PACKETIZATION;  return s; }
+const PString & OpalMediaFormat::MediaPacketizationsOption() { static PString s = PLUGINCODEC_MEDIA_PACKETIZATIONS; return s; }
 #endif
 
 
@@ -866,7 +880,7 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
                                                  unsigned ft,
                                                  unsigned cr,
                                                  time_t ts)
-  : formatName(fullName), mediaType(_mediaType)
+  : formatName(fullName), mediaType(_mediaType), forceIsTransportable(false)
 {
   codecVersionTime = ts;
   rtpPayloadType = pt;
@@ -887,8 +901,15 @@ OpalMediaFormatInternal::OpalMediaFormatInternal(const char * fullName,
     AddOption(new OpalMediaOptionUnsigned(OpalMediaFormat::ClockRateOption(), true, OpalMediaOption::AlwaysMerge, cr));
 
   // assume non-dynamic payload types are correct and do not need deconflicting
-  if (rtpPayloadType < RTP_DataFrame::DynamicBase || rtpPayloadType >= RTP_DataFrame::MaxPayloadType)
+  if (rtpPayloadType < RTP_DataFrame::DynamicBase || rtpPayloadType >= RTP_DataFrame::MaxPayloadType) {
+    if (rtpPayloadType == RTP_DataFrame::MaxPayloadType && 
+        rtpEncodingName.GetLength() > 0 &&
+        rtpEncodingName[0] == '+') {
+      forceIsTransportable = true;
+      rtpEncodingName = rtpEncodingName.Mid(1);
+    }
     return;
+  }
 
   PWaitAndSignal mutex(GetMediaFormatsListMutex());
   OpalMediaFormatList & registeredFormats = GetMediaFormatsList();
@@ -968,6 +989,27 @@ bool OpalMediaFormatInternal::ToCustomisedOptions()
 }
 
 
+bool OpalMediaFormatInternal::IsValid() const
+{
+  return rtpPayloadType < RTP_DataFrame::IllegalPayloadType && !formatName.IsEmpty();
+}
+
+
+bool OpalMediaFormatInternal::IsTransportable() const
+{
+  if (forceIsTransportable)
+    return true;
+
+  if (rtpPayloadType >= RTP_DataFrame::MaxPayloadType)
+    return false;
+
+  if (rtpPayloadType < RTP_DataFrame::LastKnownPayloadType)
+    return true;
+
+  return !rtpEncodingName.IsEmpty();
+}
+
+
 PStringToString OpalMediaFormatInternal::GetOptions() const
 {
   PWaitAndSignal m1(media_format_mutex);
@@ -1193,7 +1235,7 @@ bool OpalMediaFormatInternal::IsValidForProtocol(const PString & protocol) const
 
   // the protocol is only valid for SIP if the RTP name is not NULL
   if (protocol *= "sip")
-    return rtpEncodingName != NULL;
+    return (rtpEncodingName != NULL) || (forceIsTransportable);
 
   return true;
 }
@@ -1422,10 +1464,10 @@ OpalVideoFormatInternal::OpalVideoFormatInternal(const char * fullName,
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MinRxFrameHeightOption(),         false, OpalMediaOption::MaxMerge,    PVideoFrameInfo::SQCIFHeight,16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameWidthOption(),          false, OpalMediaOption::MinMerge,    maxFrameWidth,               16,  32767));
   AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::MaxRxFrameHeightOption(),         false, OpalMediaOption::MinMerge,    maxFrameHeight,              16,  32767));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::MinMerge,    maxBitRate,                  1000      ));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::TargetBitRateOption(),            false, OpalMediaOption::AlwaysMerge, maxBitRate,                  1000      ));
   AddOption(new OpalMediaOptionBoolean (OpalVideoFormat::RateControlEnableOption(),        false, OpalMediaOption::NoMerge,     false                                  ));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlWindowSizeOption(),    false, OpalMediaOption::NoMerge,     500,                         100, 10000));
-  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlMaxFramesSkipOption(), false, OpalMediaOption::NoMerge,     1,                           1,   10   ));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlWindowSizeOption(),    false, OpalMediaOption::NoMerge,     5000,                        100,100000));
+  AddOption(new OpalMediaOptionUnsigned(OpalVideoFormat::RateControlMaxFramesSkipOption(), false, OpalMediaOption::NoMerge,     5,                           1,   10   ));
 								
   // For video the max bit rate and frame rate is adjustable by user
   FindOption(OpalVideoFormat::MaxBitRateOption())->SetReadOnly(false);
@@ -1504,13 +1546,23 @@ OpalMediaFormatList & OpalMediaFormatList::operator-=(const OpalMediaFormatList
 }
 
 
-void OpalMediaFormatList::Remove(const PStringArray & mask)
+void OpalMediaFormatList::Remove(const PStringArray & maskList)
 {
   PINDEX i;
-  for (i = 0; i < mask.GetSize(); i++) {
-    OpalMediaFormatList::const_iterator fmt;
-    while ((fmt = FindFormat(mask[i])) != end())
-      erase(fmt);
+  for (i = 0; i < maskList.GetSize(); i++) {
+    PString mask = maskList[i];
+    if (mask.GetLength() > 1 && mask[0] == '!') {
+      mask = mask.Mid(1);
+      OpalMediaFormatList::const_iterator fmt;
+      while ((fmt = FindNotFormat(mask)) != end())
+        erase(fmt);
+    }
+    else
+    {
+      OpalMediaFormatList::const_iterator fmt;
+      while ((fmt = FindFormat(mask)) != end())
+        erase(fmt);
+    }
   }
 }
 
@@ -1595,6 +1647,19 @@ OpalMediaFormatList::const_iterator OpalMediaFormatList::FindFormat(const PStrin
   return end();
 }
 
+OpalMediaFormatList::const_iterator OpalMediaFormatList::FindNotFormat(const PString & search, const_iterator iter) const
+{
+  PStringArray wildcards = search.Tokenise('*', true);
+  if (iter == const_iterator())
+    iter = begin();
+  while (iter != end()) {
+    if (!WildcardMatch(iter->m_info->formatName, wildcards))
+      return iter;
+    ++iter;
+  }
+
+  return end();
+}
 
 void OpalMediaFormatList::Reorder(const PStringArray & order)
 {
@@ -1616,5 +1681,16 @@ void OpalMediaFormatList::Reorder(const PStringArray & order)
   AllowDeleteObjects();
 }
 
+bool OpalMediaFormatList::HasType(const OpalMediaType & type, bool mustBeTransportable) const
+{
+  OpalMediaFormatList::const_iterator format;
+  for (format = begin(); format != end(); ++format) {
+    if (format->GetMediaType() == type && (mustBeTransportable || format->IsTransportable()))
+      return true;
+  }
+
+  return false;
+}
+
 
 // End of File ///////////////////////////////////////////////////////////////
