@@ -23,9 +23,9 @@
  *
  * Contributor(s): ________________________________________.
  *
- * $Revision: 21014 $
- * $Author: rjongbloed $
- * $Date: 2008-09-16 12:36:29 +0000 (Tue, 16 Sep 2008) $
+ * $Revision: 21866 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-24 00:15:43 +0000 (Wed, 24 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -111,12 +111,8 @@ PBoolean OpalMediaStream::UpdateMediaFormat(const OpalMediaFormat & newMediaForm
     return false;
 
   // If we are source, then update the sink side, and vice versa
-  if (mediaPatch != NULL && !fromPatch) {
-    if (!mediaPatch->UpdateMediaFormat(newMediaFormat, IsSink())) {
-      PTRACE(2, "Media\tPatch did not allow media format update of " << *this);
-      return false;
-    }
-  }
+  if (mediaPatch != NULL && !fromPatch)
+    return mediaPatch->UpdateMediaFormat(newMediaFormat);
 
   if (mediaFormat != newMediaFormat)
     return mediaFormat.Merge(newMediaFormat);
@@ -324,8 +320,10 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
     unsigned oldTimestamp = timestamp;
 
     PINDEX written;
-    if (!WriteData(ptr, size, written) || (written == 0))
+    if (!WriteData(ptr, size, written) || (written == 0)) {
+      PTRACE(2, "Media\tWritePacket failed with written " << written);
       return false;
+    }
 
     // If the Write() function did not change the timestamp then use the default
     // method of fixed frame times and sizes.
@@ -346,12 +344,16 @@ PBoolean OpalMediaStream::WritePacket(RTP_DataFrame & packet)
 
 PBoolean OpalMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & length)
 {
-  if (!isOpen)
+  if (!isOpen) {
+    length = 0;
     return false;
+  }
 
   RTP_DataFrame packet(size);
-  if (!ReadPacket(packet))
+  if (!ReadPacket(packet)) {
+    length = 0;
     return false;
+  }
 
   length = packet.GetPayloadSize();
   if (length > size)
@@ -365,8 +367,10 @@ PBoolean OpalMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & length)
 
 PBoolean OpalMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX & written)
 {
-  if (!isOpen)
+  if (!isOpen) {
+    written = 0;
     return false;
+  }
 
   written = length;
   RTP_DataFrame packet(length);
@@ -585,7 +589,12 @@ OpalRTPMediaStream::OpalRTPMediaStream(OpalRTPConnection & conn,
   if (!mediaFormat.NeedsJitterBuffer())
     minAudioJitterDelay = maxAudioJitterDelay = 0;
 
-  defaultDataSize = conn.GetMaxRtpPayloadSize();
+  /* If we are a source then we should set our buffer size to the max
+     practical UDP packet size. This means we have a buffer that can accept
+     whatever the RTP sender throws at us. For sink, we set it to the
+     maximum size based on MTU (or other criteria). */
+  defaultDataSize = isSource ? 2048 : conn.GetMaxRtpPayloadSize();
+
   rtp.SetEncoding(mediaFormat.GetMediaType().GetDefinition()->GetRTPEncoding());
 }
 
@@ -657,26 +666,17 @@ PBoolean OpalRTPMediaStream::WritePacket(RTP_DataFrame & packet)
 
 }
 
+
 PBoolean OpalRTPMediaStream::SetDataSize(PINDEX dataSize)
 {
   if (dataSize <= 0)
     return false;
 
-  defaultDataSize = dataSize;
-
-  /* If we are a source then we should set our buffer size to the max
-     practical UDP packet size. This means we have a buffer that can accept
-     whatever the RTP sender throws at us. For sink, we just clamp it to the
-     maximum size based on MTU (or other criteria). */
-  PINDEX minSize = IsSource() ? 2048 : connection.GetMaxRtpPayloadSize();
-  if (defaultDataSize < minSize) {
-    PTRACE(1, "Media\tClamping RTP media stream data size from " << dataSize << " to minimum " << minSize);
-    defaultDataSize = minSize;
-  }
-
+  PTRACE(3, "Media\tRTP data size cannot be changed to " << dataSize << ", fixed at " << defaultDataSize);
   return true;
 }
 
+
 PBoolean OpalRTPMediaStream::IsSynchronous() const
 {
   return IsSource() && !mediaFormat.NeedsJitterBuffer();
@@ -752,8 +752,10 @@ PBoolean OpalRawMediaStream::ReadData(BYTE * buffer, PINDEX size, PINDEX & lengt
 
 PBoolean OpalRawMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDEX & written)
 {
-  if (!isOpen)
+  if (!isOpen) {
+    PTRACE(1, "Media\tTried to write to closed media stream");
     return false;
+  }
 
   written = 0;
 
@@ -762,8 +764,10 @@ PBoolean OpalRawMediaStream::WriteData(const BYTE * buffer, PINDEX length, PINDE
     return false;
   }
   
-  if (!IsOpen() || channel == NULL)
+  if (!IsOpen() || channel == NULL) {
+    PTRACE(1, "Media\tTried to write to media stream with no channel");
     return false;
+  }
 
   if (buffer != NULL && length != 0) {
     if (!channel->Write(buffer, length))
