@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21749 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-10 02:20:12 +0000 (Wed, 10 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -142,6 +142,9 @@ class SIPEndPoint_C : public SIPEndPoint
       bool reSubscribing,           ///< If subscribing then indication was refeshing subscription
       SIP_PDU::StatusCodes reason   ///< Status of subscription
     );
+    virtual void OnDialogInfoReceived(
+      const SIPDialogNotification & info  ///< Information on dialog state change
+    );
 
   private:
     OpalManager_C & m_manager;
@@ -617,17 +620,48 @@ void SIPEndPoint_C::OnSubscriptionStatus(const PString & eventPackage,
   SIPEndPoint::OnSubscriptionStatus(eventPackage, uri, wasSubscribing, reSubscribing, reason);
 
   if (reason == SIP_PDU::Successful_OK && !reSubscribing) {
-    if (eventPackage == SIPSubscribe::GetEventPackageName(SIPSubscribe::MessageSummary)) {
+    if (SIPEventPackage(SIPSubscribe::MessageSummary) == eventPackage) {
       OpalMessageBuffer message(OpalIndMessageWaiting);
       SET_MESSAGE_STRING(message, m_param.m_messageWaiting.m_party, uri.AsString());
       SET_MESSAGE_STRING(message, m_param.m_messageWaiting.m_extraInfo, wasSubscribing ? "SUBSCRIBED" : "UNSUBSCRIBED");
       PTRACE(4, "OpalC API\tOnSubscriptionStatus - MWI: party=\"" << message->m_param.m_messageWaiting.m_party
-                                            << "\" info=" << message->m_param.m_messageWaiting.m_extraInfo);
+                                              << "\" info=" << message->m_param.m_messageWaiting.m_extraInfo);
+      m_manager.PostMessage(message);
+    }
+    else if (SIPEventPackage(SIPSubscribe::Dialog) == eventPackage) {
+      OpalMessageBuffer message(OpalIndLineAppearance);
+      SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_line, uri.AsString());
+      message->m_param.m_lineAppearance.m_state = wasSubscribing ? OpalLineSubcribed : OpalLineUnsubcribed;
+      PTRACE(4, "OpalC API\tOnSubscriptionStatus - LineAppearance: line=\"" << message->m_param.m_lineAppearance.m_line);
       m_manager.PostMessage(message);
     }
   }
+}
 
 
+void SIPEndPoint_C::OnDialogInfoReceived(const SIPDialogNotification & info)
+{
+  SIPEndPoint::OnDialogInfoReceived(info);
+
+  OpalMessageBuffer message(OpalIndLineAppearance);
+  SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_line, info.m_entity);
+  message->m_param.m_lineAppearance.m_state = (OpalLineAppearanceStates)info.m_state;
+  message->m_param.m_lineAppearance.m_appearance = info.m_local.m_appearance;
+
+  if (info.m_initiator) {
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_callId, info.m_callId+";to-tag="+info.m_local.m_dialogTag+";from-tag="+info.m_remote.m_dialogTag);
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyA, info.m_local.m_URI);
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyB, info.m_remote.m_URI);
+  }
+  else {
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_callId, info.m_callId+";to-tag="+info.m_remote.m_dialogTag+";from-tag="+info.m_local.m_dialogTag);
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyA, info.m_remote.m_URI);
+    SET_MESSAGE_STRING(message, m_param.m_lineAppearance.m_partyB, info.m_local.m_URI);
+  }
+
+  PTRACE(4, "OpalC API\tOnDialogInfoReceived: entity=\"" << message->m_param.m_lineAppearance.m_line
+                                          << "\" callId=" << message->m_param.m_lineAppearance.m_callId);
+  m_manager.PostMessage(message);
 }
 
 
@@ -1201,22 +1235,17 @@ void OpalManager_C::HandleRegistration(const OpalMessage & command, OpalMessageB
       return;
     }
 
-    PString aor = command.m_param.m_registrationInfo.m_identifier;
-    if (aor.Find('@') == P_MAX_INDEX) {
-      if (aor.IsEmpty())
-        aor = PProcess::Current().GetUserName();
-      aor += '@';
-      aor += command.m_param.m_registrationInfo.m_hostName;
-    }
-
     if (command.m_param.m_registrationInfo.m_timeToLive == 0) {
       if (!sip->Unregister(command.m_param.m_registrationInfo.m_identifier))
         response.SetError("Failed to initiate SIP unregistration.");
     }
     else {
+      PString aor;
+
       if (m_apiVersion < 13 || command.m_param.m_registrationInfo.m_eventPackage == NULL) {
         SIPRegister::Params regParams;
-        regParams.m_addressOfRecord = aor;
+        regParams.m_addressOfRecord = command.m_param.m_registrationInfo.m_identifier;
+        regParams.m_registrarAddress = command.m_param.m_registrationInfo.m_hostName;
         regParams.m_authID = command.m_param.m_registrationInfo.m_authUserName;
         regParams.m_password = command.m_param.m_registrationInfo.m_password;
         regParams.m_realm = command.m_param.m_registrationInfo.m_adminEntity;
@@ -1224,7 +1253,7 @@ void OpalManager_C::HandleRegistration(const OpalMessage & command, OpalMessageB
         if (m_apiVersion >= 7 && command.m_param.m_registrationInfo.m_restoreTime > 0)
           regParams.m_restoreTime = command.m_param.m_registrationInfo.m_restoreTime;
 
-        if (sip->Register(regParams))
+        if (sip->Register(regParams, aor))
           SET_MESSAGE_STRING(response, m_param.m_registrationInfo.m_identifier, aor);
         else
           response.SetError("Failed to initiate SIP registration.");
@@ -1241,7 +1270,8 @@ void OpalManager_C::HandleRegistration(const OpalMessage & command, OpalMessageB
           subParams.m_eventPackage = command.m_param.m_registrationInfo.m_eventPackage;
         }
 
-        subParams.m_targetAddress = aor;
+        subParams.m_addressOfRecord = command.m_param.m_registrationInfo.m_identifier;
+        subParams.m_agentAddress = command.m_param.m_registrationInfo.m_hostName;
         subParams.m_authID = command.m_param.m_registrationInfo.m_authUserName;
         subParams.m_password = command.m_param.m_registrationInfo.m_password;
         subParams.m_realm = command.m_param.m_registrationInfo.m_adminEntity;
@@ -1252,7 +1282,7 @@ void OpalManager_C::HandleRegistration(const OpalMessage & command, OpalMessageB
                                                : (unsigned)command.m_param.m_registrationInfo.m_eventPackage; // Backward compatibility
 #endif
         subParams.m_restoreTime = command.m_param.m_registrationInfo.m_restoreTime;
-        bool ok = sip->Subscribe(subParams);
+        bool ok = sip->Subscribe(subParams, aor);
         if (m_apiVersion >= 13) {
           if (ok)
             SET_MESSAGE_STRING(response, m_param.m_registrationInfo.m_identifier, aor);
