@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21760 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-10 05:58:09 +0000 (Wed, 10 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -42,6 +42,9 @@
 #include <opal/mediastrm.h>
 #include <opal/transcoders.h>
 
+#if OPAL_VIDEO
+#include <codec/vidcodec.h>
+#endif
 
 #define new PNEW
 
@@ -316,12 +319,6 @@ OpalMediaPatch::Sink::Sink(OpalMediaPatch & p, const OpalMediaStreamPtr & s)
   , writeSuccessful(true)
 #if OPAL_VIDEO
   , rcEnabled(false)
-  , rcByteRate(0)
-  , rcWindowSize(0)
-  , rcMaxConsecutiveFramesSkip(0)
-  , rcConsecutiveFramesSkipped(0)
-  , rcLastTime(PTimer::Tick())
-  , rcTotalSize(0)
 #endif
 {
   intermediateFrames.Append(new RTP_DataFrame);
@@ -425,27 +422,35 @@ void OpalMediaPatch::SetCommandNotifier(const PNotifier & notifier, PBoolean fro
   }
 }
 
+
+bool OpalMediaPatch::OnPatchStart()
+{
+  source.OnPatchStart();
+
+  if (source.IsSynchronous())
+    return false;
+
+  bool isAudio = source.GetMediaFormat().GetMediaType() == OpalMediaType::Audio();
+
+  PReadWaitAndSignal mutex(inUse);
+
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->IsSynchronous()) {
+      if (isAudio) 
+        source.EnableJitterBuffer();
+      return false;
+    }
+  }
+	
+  return true;
+}
+
+
 void OpalMediaPatch::Main()
 {
   PTRACE(4, "Patch\tThread started for " << *this);
 	
-  bool isAudio = source.GetMediaFormat().GetMediaType() == OpalMediaType::Audio();
-
-  inUse.StartRead();
-  source.OnPatchStart();
-  PBoolean isSynchronous = source.IsSynchronous();
-  if (!source.IsSynchronous()) {
-    for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
-      if (s->stream->IsSynchronous()) {
-        if (isAudio) 
-          source.EnableJitterBuffer();
-        isSynchronous = PTrue;
-        break;
-      }
-    }
-  }
-	
-  inUse.EndRead();
+  bool asynchronous = OnPatchStart();
 
   RTP_DataFrame sourceFrame(source.GetDataSize());
   sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
@@ -469,7 +474,7 @@ void OpalMediaPatch::Main()
     // Don't starve the CPU if we have idle frames and the no source or destination is synchronous.
     // Note that performing a Yield is not good enough, as the media patch threads are
     // high priority and will consume all available CPU if allowed.
-    if (!isSynchronous)
+    if (asynchronous)
       PThread::Sleep(5);
   }
 
@@ -488,7 +493,7 @@ bool OpalMediaPatch::DispatchFrame(RTP_DataFrame & frame)
     if (s->WriteFrame(frame))
       written = true;
     else {
-      PTRACE(2, "Patch\tWritePacket failed");
+      PTRACE(2, "Patch\tWriteFrame failed");
     }
   }
 
@@ -513,7 +518,7 @@ bool OpalMediaPatch::Sink::UpdateMediaFormat(const OpalMediaFormat & mediaFormat
          stream->UpdateMediaFormat(primaryCodec->GetInputFormat(), true);
 
 #if OPAL_VIDEO
-  SetRateControlParameters(stream->GetMediaFormat());
+    SetRateControlParameters(stream->GetMediaFormat());
 #endif
 
   PTRACE(3, "Patch\tUpdated Sink: format=" << mediaFormat << " ok=" << ok);
@@ -561,6 +566,7 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
   if ((pt != codec.GetPayloadType(true)) && !codec.AcceptOtherPayloads()) {
     PTRACE(4, "Patch\tRemoving frame with mismatched payload type " << pt << " - should be " << codec.GetPayloadType(true));
     frame.SetPayloadSize(0);   // remove the payload because the transcoder has indicated it won't understand it
+    frame.SetPayloadType(codec.GetPayloadType(true)); // Reset pt so if get silence frames from jitter buffer, they don't cause errors
     return true;
   }
 
@@ -575,60 +581,29 @@ static bool CannotTranscodeFrame(const OpalTranscoder & codec, RTP_DataFrame & f
 void OpalMediaPatch::Sink::SetRateControlParameters(const OpalMediaFormat & mediaFormat)
 {
   unsigned targetBitRate = mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption());
-  rcEnabled = mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption());
-  rcWindowSize = mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlWindowSizeOption());
-  rcMaxConsecutiveFramesSkip = mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlMaxFramesSkipOption());
+  rcEnabled = (mediaFormat.GetMediaType() == OpalMediaType::Video()) &&
+              mediaFormat != OpalYUV420P &&
+              mediaFormat.GetOptionBoolean(OpalVideoFormat::RateControlEnableOption());
 
-  if (((unsigned) (targetBitRate / 8)) != rcByteRate) {
-    rcByteRate = (unsigned) (targetBitRate / 8);
-
-    std::list<FrameInfo>::iterator iter = frameInfoList.begin();
-    while (iter != frameInfoList.end()) {
-      rcTotalSize -= iter->size;
-      frameInfoList.erase(iter);
-      iter = frameInfoList.begin();
-    }
-  }
-
-  PTRACE(3, "Patch\tSet rate control from format " << mediaFormat
-         << ", bitrate=" << targetBitRate
-         << ", rate control=" << rcEnabled
-         << ", window=" << rcWindowSize);
+  if (rcEnabled) 
+    rateController.Open(
+                        targetBitRate,
+                        mediaFormat.GetOptionInteger(OpalVideoFormat::FrameTimeOption(), -1),
+                        mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlWindowSizeOption()),
+                        mediaFormat.GetOptionInteger(OpalVideoFormat::RateControlMaxFramesSkipOption())
+                        );
 }
 
 
-bool OpalMediaPatch::Sink::RateControlExceeded(const PTimeInterval & currentTime)
+bool OpalMediaPatch::Sink::RateControlExceeded()
 {
-  if (!rcEnabled)
+  if (!rcEnabled || !rateController.SkipFrame()) 
     return false;
 
-  std::list<FrameInfo>::iterator iter = frameInfoList.begin();
-  while (iter != frameInfoList.end()) {
-    if ((currentTime - iter->time) < rcWindowSize) {
-      rcLastTime = iter->time;
-      break;
-    }
-
-    rcTotalSize -= iter->size;
-    frameInfoList.erase(iter);
-    iter = frameInfoList.begin();
-  }
-
-  unsigned passedTime = (unsigned)(currentTime - rcLastTime).GetMilliSeconds();
-  PTRACE(5, "Patch\tRTP packets,  accumulated: " << rcTotalSize
-         << " list: " << frameInfoList.size()
-         << "over Time " << passedTime
-         << " byterate " << rcTotalSize * 1000 / passedTime
-         << " <= " << rcByteRate);
-  if (rcTotalSize * 1000 / passedTime < rcByteRate || rcConsecutiveFramesSkipped >= rcMaxConsecutiveFramesSkip) {
-    rcConsecutiveFramesSkipped = 0;
-    return false;
-  }
-
   PTRACE(4, "Patch\tRate controller skipping frame.");
-    ++rcConsecutiveFramesSkipped;
-    return true;
+  return true;
 }
+
 #endif
 
 
@@ -638,11 +613,11 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     return false;
   
 #if OPAL_VIDEO
-  FrameInfo frameInfo;
-  frameInfo.time = PTimer::Tick();
-  frameInfo.size = 0;
-  if (RateControlExceeded(frameInfo.time))
+  if (RateControlExceeded())
     return true;
+
+  PInt64 totalPayloadSize = 0;
+  int frameCount = 0;
 #endif
 
   if (primaryCodec == NULL)
@@ -663,7 +638,8 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       if (!stream->WritePacket(*interFrame))
         return (writeSuccessful = false);
 #if OPAL_VIDEO
-      frameInfo.size += interFrame->GetPayloadSize() + interFrame->GetHeaderSize();
+      totalPayloadSize += interFrame->GetPayloadSize() + interFrame->GetHeaderSize();
+      frameCount++;
 #endif
       sourceFrame.SetTimestamp(interFrame->GetTimestamp());
       continue;
@@ -685,17 +661,16 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       if (!stream->WritePacket(*finalFrame))
         return (writeSuccessful = false);
 #if OPAL_VIDEO
-      frameInfo.size += finalFrame->GetPayloadSize() + finalFrame->GetHeaderSize();
+      totalPayloadSize += finalFrame->GetPayloadSize() + finalFrame->GetHeaderSize();
+      frameCount++;
 #endif
       sourceFrame.SetTimestamp(finalFrame->GetTimestamp());
     }
   }
 
 #if OPAL_VIDEO
-  if (rcEnabled && frameInfo.size > 0) {
-    frameInfoList.push_back(frameInfo);
-    rcTotalSize += frameInfo.size;
-  }
+  if (rcEnabled)
+    rateController.AddFrame(totalPayloadSize, frameCount);
 #endif
 
   return true;
@@ -734,4 +709,3 @@ PBoolean OpalPassiveMediaPatch::PushFrame(RTP_DataFrame & frame)
 }
 
 
-/////////////////////////////////////////////////////////////////////////////
