@@ -28,9 +28,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21599 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-23 12:42:25 +0000 (Sun, 23 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -95,7 +95,7 @@ OpalTransportAddress::OpalTransportAddress(const PString & str,
 
 
 OpalTransportAddress::OpalTransportAddress(const PIPSocket::Address & addr, WORD port, const char * proto)
-  : PString(addr.AsString())
+  : PString(addr.AsString(true))
 {
   SetInternalTransport(port, proto);
 }
@@ -132,8 +132,8 @@ PBoolean OpalTransportAddress::IsCompatible(const OpalTransportAddress & address
   if (IsEmpty() || address.IsEmpty())
     return PTrue;
 
-  PCaselessString myPrefix = Left(Find('$'));
-  PCaselessString theirPrefix = address.Left(address.Find('$'));
+  PCaselessString myPrefix = GetProto();
+  PCaselessString theirPrefix = address.GetProto();
   return myPrefix == theirPrefix ||
         (myPrefix    == IpPrefix && (theirPrefix == TcpPrefix || theirPrefix == UdpPrefix 
 #if OPAL_PTLIB_SSL
@@ -209,6 +209,11 @@ void OpalTransportAddress::SetInternalTransport(WORD port, const char * proto)
   if (transport == NULL)
     return;
 
+  // Get port, even if string is bracketed, i.e. udp$[2001...]:1720
+  PINDEX rbracket = Find(']');
+  if( rbracket != P_MAX_INDEX )
+	  dollar = rbracket + 1;
+
   if (port != 0 && Find(':', dollar) == P_MAX_INDEX) {
     PINDEX end = GetLength();
     if (GetAt(end-1) == '+')
@@ -314,7 +319,7 @@ PString OpalInternalIPTransport::GetHostName(const OpalTransportAddress & addres
 
   PIPSocket::Address ip;
   if (ip.FromString(host))
-    return ip.AsString();
+    return ip.AsString(true);
 
   return host;
 }
@@ -337,7 +342,7 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     port = 0;
   else {
     if (!service) {
-      PString proto = address.Left(address.Find('$'));
+      PCaselessString proto = address.GetProto();
       if (proto *= "ip")
         proto = "tcp";
       port = PIPSocket::GetPortByService(proto, service);
@@ -348,7 +353,11 @@ PBoolean OpalInternalIPTransport::GetIpAndPort(const OpalTransportAddress & addr
     }
   }
 
-  if (host[0] == '*' || host == "0.0.0.0") {
+  if (host[0] == '*' || host == "0.0.0.0"
+#ifdef P_HAS_IPV6
+	  || host == ":" || host == "::"  || host == "[::]"
+#endif
+	  ) {
     ip = PIPSocket::GetDefaultIpAny();
     return PTrue;
   }
@@ -512,7 +521,7 @@ OpalTransportAddress OpalListenerIP::GetLocalAddress(const OpalTransportAddress
 
   // If specifically bound to interface use that
   if (!localAddress.IsAny())
-    addr = localAddress.AsString();
+    addr = localAddress.AsString(true);
   else {
     // If bound to all, then use '*' unless a preferred address is specified
     addr = "*";
@@ -524,7 +533,7 @@ OpalTransportAddress OpalListenerIP::GetLocalAddress(const OpalTransportAddress
       if (PIPSocket::GetInterfaceTable(interfaces)) {
         for (PINDEX i = 0; i < interfaces.GetSize(); i++) {
           if (interfaces[i].GetAddress() == ip) {
-            addr = ip.AsString();
+            addr = ip.AsString(true);
             break;
           }
         }
@@ -583,7 +592,8 @@ PBoolean OpalListenerTCP::Open(const PNotifier & theAcceptHandler, ThreadMode mo
   if (listener.Listen(localAddress, 10, listenerPort, exclusiveListener ? PSocket::AddressIsExclusive : PSocket::CanReuseAddress))
     return StartThread(theAcceptHandler, mode);
 
-  PTRACE(1, "Listen\tOpen (" << (exclusiveListener ? "EXCLUSIVE" : "REUSEADDR") << ") on " << localAddress << ':' << listener.GetPort()
+  PTRACE(1, "Listen\tOpen (" << (exclusiveListener ? "EXCLUSIVE" : "REUSEADDR") << ") on "
+         << localAddress.AsString(true) << ':' << listener.GetPort()
          << " failed: " << listener.GetErrorText());
   return false;
 }
@@ -629,7 +639,6 @@ OpalTransport * OpalListenerTCP::Accept(const PTimeInterval & timeout)
 }
 
 
-
 OpalTransport * OpalListenerTCP::CreateTransport(const OpalTransportAddress & localAddress,
                                                  const OpalTransportAddress & remoteAddress) const
 {
@@ -743,7 +752,7 @@ OpalTransport * OpalListenerUDP::CreateTransport(const OpalTransportAddress & lo
 
   PString iface;
   if (localAddress.GetIpAddress(addr))
-    iface = addr.AsString();
+    iface = addr.AsString(true);
 
   return new OpalTransportUDP(endpoint, PBYTEArray(), listenerBundle, iface, PIPSocket::GetDefaultIpAny(), 0);
 }
@@ -882,7 +891,7 @@ OpalTransportIP::OpalTransportIP(OpalEndPoint & end,
                                  WORD port)
   : OpalTransport(end),
     localAddress(binding),
-    remoteAddress(0)
+    remoteAddress(PIPSocket::Address::GetAny(binding.GetVersion()))
 {
   localPort = port;
   remotePort = 0;
@@ -983,7 +992,7 @@ PBoolean OpalTransportTCP::Connect()
   WORD firstPort = localPort;
   for (;;) {
     PTRACE(4, "OpalTCP\tConnecting to "
-           << remoteAddress << ':' << remotePort
+           << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
     if (socket->Connect(localPort, remoteAddress))
       break;
@@ -991,7 +1000,7 @@ PBoolean OpalTransportTCP::Connect()
     int errnum = socket->GetErrorNumber();
     if (localPort == 0 || (errnum != EADDRINUSE && errnum != EADDRNOTAVAIL)) {
       PTRACE(1, "OpalTCP\tCould not connect to "
-                << remoteAddress << ':' << remotePort
+                << remoteAddress.AsString(true) << ':' << remotePort
                 << " (local port=" << localPort << ") - "
                 << socket->GetErrorText() << '(' << errnum << ')');
       return SetErrorValues(socket->GetErrorCode(), errnum);
@@ -1103,8 +1112,8 @@ PBoolean OpalTransportTCP::OnOpen()
 #endif
 
   PTRACE(3, "OpalTCP\tStarted connection to "
-         << remoteAddress << ':' << remotePort
-         << " (if=" << localAddress << ':' << localPort << ')');
+         << remoteAddress.AsString(true) << ':' << remotePort
+         << " (if=" << localAddress.AsString(true) << ':' << localPort << ')');
 
   return PTrue;
 }
@@ -1152,7 +1161,7 @@ OpalTransportUDP::OpalTransportUDP(OpalEndPoint & ep,
   socket->GetLocal(localAddress, localPort, !manager.IsLocalAddress(remoteAddress));
   Open(socket);
 
-  PTRACE(3, "OpalUDP\tBinding to interface: " << localAddress << ':' << localPort);
+  PTRACE(3, "OpalUDP\tBinding to interface: " << localAddress.AsString(true) << ':' << localPort);
 }
 
 
@@ -1182,11 +1191,11 @@ PBoolean OpalTransportUDP::Connect()
     return PFalse;
 
   if (remoteAddress.IsAny() || remoteAddress.IsBroadcast()) {
-    remoteAddress = PIPSocket::Address::GetBroadcast();
+	  remoteAddress = PIPSocket::Address::GetBroadcast(remoteAddress.GetVersion());
     PTRACE(3, "OpalUDP\tBroadcast connect to port " << remotePort);
   }
   else {
-    PTRACE(3, "OpalUDP\tStarted connect to " << remoteAddress << ':' << remotePort);
+    PTRACE(3, "OpalUDP\tStarted connect to " << remoteAddress.AsString(true) << ':' << remotePort);
   }
 
   if (PAssertNULL(writeChannel) == NULL)
@@ -1470,7 +1479,7 @@ PBoolean OpalTransportTCPS::Connect()
   WORD firstPort = localPort;
   for (;;) {
     PTRACE(4, "OpalTCPS\tConnecting to "
-           << remoteAddress << ':' << remotePort
+           << remoteAddress.AsString(true) << ':' << remotePort
            << " (local port=" << localPort << ')');
     if (socket->Connect(localPort, remoteAddress))
       break;
@@ -1478,7 +1487,7 @@ PBoolean OpalTransportTCPS::Connect()
     int errnum = socket->GetErrorNumber();
     if (localPort == 0 || (errnum != EADDRINUSE && errnum != EADDRNOTAVAIL)) {
       PTRACE(1, "OpalTCPS\tCould not connect to "
-                << remoteAddress << ':' << remotePort
+                << remoteAddress.AsString(true) << ':' << remotePort
                 << " (local port=" << localPort << ") - "
                 << socket->GetErrorText() << '(' << errnum << ')');
       return SetErrorValues(socket->GetErrorCode(), errnum);
@@ -1544,8 +1553,8 @@ PBoolean OpalTransportTCPS::OnOpen()
 #endif
 
   PTRACE(3, "OpalTCPS\tStarted connection to "
-         << remoteAddress << ':' << remotePort
-         << " (if=" << localAddress << ':' << localPort << ')');
+         << remoteAddress.AsString(true) << ':' << remotePort
+         << " (if=" << localAddress.AsString(true) << ':' << localPort << ')');
 
   return PTrue;
 }
