@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21035 $
- * $Author: rjongbloed $
- * $Date: 2008-09-17 07:57:58 +0000 (Wed, 17 Sep 2008) $
+ * $Revision: 21835 $
+ * $Author: hfriederich $
+ * $Date: 2008-12-18 09:00:44 +0000 (Thu, 18 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -48,6 +48,7 @@
 
 #define new PNEW
 
+#define BAD_TRANSMIT_TIME_MAX 10    //  maximum of seconds of transmit fails before session is killed
 
 const unsigned SecondsFrom1900to1970 = (70*365+17)*24*60*60U;
 
@@ -464,7 +465,7 @@ void RTP_UserData::OnRxStatistics(const RTP_Session & /*session*/) const
 {
 }
 
-void RTP_UserData::OnClearCall(const RTP_Session & /*session*/)
+void RTP_UserData::SessionFailing(RTP_Session & /*session*/)
 {
 }
 
@@ -482,13 +483,11 @@ void RTP_UserData::OnTxIntraFrameRequest(const RTP_Session & /*session*/) const
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_Session::RTP_Session(const Params & params)
-: canonicalName(PProcess::Current().GetUserName()),
-  toolName(PProcess::Current().GetName()),
-  reportTimeInterval(0, 12),  // Seconds
-  reportTimer(reportTimeInterval)
-#if OPAL_RTP_AGGREGATE
-  , aggregator(params.aggregator)
-#endif
+  : canonicalName(PProcess::Current().GetUserName())
+  , toolName(PProcess::Current().GetName())
+  , reportTimeInterval(0, 12)  // Seconds
+  , reportTimer(reportTimeInterval)
+  , failed(false)
 {
   PAssert(params.id > 0 && params.id < 256, PInvalidParameter);
   sessionID = (BYTE)params.id;
@@ -518,34 +517,7 @@ RTP_Session::RTP_Session(const Params & params)
   lastRRSequenceNumber = 0;
   consecutiveOutOfOrderPackets = 0;
 
-  packetsSent = 0;
-  rtcpPacketsSent = 0;
-  octetsSent = 0;
-  packetsReceived = 0;
-  octetsReceived = 0;
-  packetsLost = 0;
-  packetsOutOfOrder = 0;
-  averageSendTime = 0;
-  maximumSendTime = 0;
-  minimumSendTime = 0;
-  averageReceiveTime = 0;
-  maximumReceiveTime = 0;
-  minimumReceiveTime = 0;
-  jitterLevel = 0;
-  maximumJitterLevel = 0;
-  markerRecvCount = 0;
-  markerSendCount = 0;
-
-  txStatisticsCount = 0;
-  rxStatisticsCount = 0;
-  averageSendTimeAccum = 0;
-  maximumSendTimeAccum = 0;
-  minimumSendTimeAccum = 0xffffffff;
-  averageReceiveTimeAccum = 0;
-  maximumReceiveTimeAccum = 0;
-  minimumReceiveTimeAccum = 0xffffffff;
-  packetsLostSinceLastRR = 0;
-  lastTransitTime = 0;
+  ClearStatistics();
 
   lastReceivedPayloadType = RTP_DataFrame::IllegalPayloadType;
 
@@ -585,6 +557,40 @@ RTP_Session::~RTP_Session()
   delete m_encodingHandler;
 }
 
+
+void RTP_Session::ClearStatistics()
+{
+  packetsSent = 0;
+  rtcpPacketsSent = 0;
+  octetsSent = 0;
+  packetsReceived = 0;
+  octetsReceived = 0;
+  packetsLost = 0;
+  packetsOutOfOrder = 0;
+  averageSendTime = 0;
+  maximumSendTime = 0;
+  minimumSendTime = 0;
+  averageReceiveTime = 0;
+  maximumReceiveTime = 0;
+  minimumReceiveTime = 0;
+  jitterLevel = 0;
+  maximumJitterLevel = 0;
+  markerRecvCount = 0;
+  markerSendCount = 0;
+
+  txStatisticsCount = 0;
+  rxStatisticsCount = 0;
+  averageSendTimeAccum = 0;
+  maximumSendTimeAccum = 0;
+  minimumSendTimeAccum = 0xffffffff;
+  averageReceiveTimeAccum = 0;
+  maximumReceiveTimeAccum = 0;
+  minimumReceiveTimeAccum = 0xffffffff;
+  packetsLostSinceLastRR = 0;
+  lastTransitTime = 0;
+}
+
+
 void RTP_Session::SendBYE()
 {
   {
@@ -695,11 +701,7 @@ void RTP_Session::SetJitterBufferSize(unsigned minJitterDelay,
       jitter->SetDelay(minJitterDelay, maxJitterDelay);
     else
       jitter = new RTP_JitterBuffer(*this, minJitterDelay, maxJitterDelay, timeUnits, stackSize);
-    jitter->Resume(
-#if OPAL_RTP_AGGREGATE
-      aggregator
-#endif
-    );
+    jitter->Resume();
   }
 }
 
@@ -1042,33 +1044,36 @@ RTP_Session::SendReceiveStatus RTP_Session::Internal_OnReceiveData(RTP_DataFrame
   if (!SendReport())
     return e_AbortTransport;
 
-  if (rxStatisticsCount < rxStatisticsInterval)
-    return e_ProcessPacket;
+  if (rxStatisticsCount >= rxStatisticsInterval) {
 
-  rxStatisticsCount = 0;
+    rxStatisticsCount = 0;
 
-  averageReceiveTime = averageReceiveTimeAccum/rxStatisticsInterval;
-  maximumReceiveTime = maximumReceiveTimeAccum;
-  minimumReceiveTime = minimumReceiveTimeAccum;
+    averageReceiveTime = averageReceiveTimeAccum/rxStatisticsInterval;
+    maximumReceiveTime = maximumReceiveTimeAccum;
+    minimumReceiveTime = minimumReceiveTimeAccum;
 
-  averageReceiveTimeAccum = 0;
-  maximumReceiveTimeAccum = 0;
-  minimumReceiveTimeAccum = 0xffffffff;
+    averageReceiveTimeAccum = 0;
+    maximumReceiveTimeAccum = 0;
+    minimumReceiveTimeAccum = 0xffffffff;
 
-  PTRACE(4, "RTP\tSession " << sessionID << ", receive statistics:"
-            " packets=" << packetsReceived <<
-            " octets=" << octetsReceived <<
-            " lost=" << packetsLost <<
-            " tooLate=" << GetPacketsTooLate() <<
-            " order=" << packetsOutOfOrder <<
-            " avgTime=" << averageReceiveTime <<
-            " maxTime=" << maximumReceiveTime <<
-            " minTime=" << minimumReceiveTime <<
-            " jitter=" << (jitterLevel >> 7) <<
-            " maxJitter=" << (maximumJitterLevel >> 7));
+    PTRACE(4, "RTP\tSession " << sessionID << ", receive statistics:"
+              " packets=" << packetsReceived <<
+              " octets=" << octetsReceived <<
+              " lost=" << packetsLost <<
+              " tooLate=" << GetPacketsTooLate() <<
+              " order=" << packetsOutOfOrder <<
+              " avgTime=" << averageReceiveTime <<
+              " maxTime=" << maximumReceiveTime <<
+              " minTime=" << minimumReceiveTime <<
+              " jitter=" << (jitterLevel >> 7) <<
+              " maxJitter=" << (maximumJitterLevel >> 7));
 
-  if (userData != NULL)
-    userData->OnRxStatistics(*this);
+    if (userData != NULL)
+      userData->OnRxStatistics(*this);
+  }
+
+  for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) 
+    f->notifier(frame, (INT)this);
 
   return e_ProcessPacket;
 }
@@ -1464,6 +1469,17 @@ PBoolean RTP_Session::WriteOOBData(RTP_DataFrame &, bool)
   return true;
 }
 
+void RTP_Session::AddFilter(const PNotifier & filter)
+{
+  // ensures that a filter is added only once
+  for (PList<Filter>::iterator f = filters.begin(); f != filters.end(); ++f) {
+    if (f->notifier == filter)
+      return;
+  }
+  filters.Append(new Filter(filter));
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 static void SetMinBufferSize(PUDPSocket & sock, int buftype)
@@ -1914,15 +1930,16 @@ RTP_Session::SendReceiveStatus RTP_UDP::ReadDataOrControlPDU(BYTE * framePtr,
   switch (socket.GetErrorNumber()) {
     case ECONNRESET :
     case ECONNREFUSED :
-      if (++badTransmitCounter < 25) { 
-        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " port on remote not ready.");
-        return RTP_Session::e_IgnorePacket;
-      }
+      PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " port on remote not ready.");
+      if (++badTransmitCounter == 1) 
+        badTransmitStart = PTime();
       else {
-        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " too many transmit fails - killing call");
-        userData->OnClearCall(*this);
-        return RTP_Session::e_AbortTransport;
+        if ((PTime()- badTransmitStart).GetMilliSeconds() < BAD_TRANSMIT_TIME_MAX * 1000)
+          return RTP_Session::e_IgnorePacket;
+        PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName << " " << BAD_TRANSMIT_TIME_MAX << "s of transmit fails - informing connection");
+        userData->SessionFailing(*this);
       }
+      return RTP_Session::e_IgnorePacket;
 
     case EMSGSIZE :
       PTRACE(2, "RTP_UDP\tSession " << sessionID << ", " << channelName
@@ -2147,6 +2164,8 @@ void RTP_Session::SetEncoding(const PString & newEncoding)
     m_encodingHandler = newHandler;
   }
 
+  ClearStatistics();
+
   EncodingLock(*this)->OnStart(*this);
 }
 
