@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21860 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-12-23 02:24:22 +0000 (Tue, 23 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -59,23 +59,23 @@
 //  the following functions bind the media type factory to the SDP format types
 //
 
-PString OpalMediaType::GetSDPFromFromMediaType(const OpalMediaType & type)
-{
-  OpalMediaTypeDefinition * defn = type.GetDefinition();
-  if (defn == NULL)
-    return PString();
-  return defn->GetSDPType();
-}
-
-OpalMediaType OpalMediaType::GetMediaTypeFromSDP(const std::string & sdp)
+OpalMediaType OpalMediaType::GetMediaTypeFromSDP(const std::string & sdp, const std::string & transport)
 {
   OpalMediaTypeFactory::KeyList_T mediaTypes = OpalMediaTypeFactory::GetKeyList();
   OpalMediaTypeFactory::KeyList_T::iterator r;
+
   for (r = mediaTypes.begin(); r != mediaTypes.end(); ++r) {
     if (OpalMediaType::GetDefinition(*r)->GetSDPType() == sdp)
       return OpalMediaType(*r);
   }
 
+  std::string s = sdp +"|" + transport;
+
+  for (r = mediaTypes.begin(); r != mediaTypes.end(); ++r) {
+    if (OpalMediaType::GetDefinition(*r)->GetSDPType() == s)
+      return OpalMediaType(*r);
+  }
+
   return OpalMediaType();
 }
 
@@ -143,7 +143,7 @@ static PString GetConnectAddressString(const OpalTransportAddress & address)
 
   PIPSocket::Address ip;
   if (!address.IsEmpty() && address.GetIpAddress(ip) && ip.IsValid())
-    str << "IN IP" << ip.GetVersion() << ' ' << ip;
+    str << "IN IP" << ip.GetVersion() << ' ' << ip.AsString(PTrue);
   else
     str << "IN IP4 0.0.0.0";
 
@@ -157,34 +157,16 @@ SDPMediaFormat::SDPMediaFormat(RTP_DataFrame::PayloadTypes pt, const char * _nam
   : payloadType(pt)
   , clockRate(0)
   , encodingName(_name)
-  , nteSet(PTrue)
-  , nseSet(PTrue)
 {
-  if (encodingName == OpalRFC2833.GetEncodingName())
-    AddNTEString("0-15,32-49");
-#if OPAL_T38_CAPABILITY
-  else if (encodingName == OpalCiscoNSE.GetEncodingName())
-    AddNSEString("192,193");
-#endif
 }
 
 
-SDPMediaFormat::SDPMediaFormat(const OpalMediaFormat & fmt,
-                               RTP_DataFrame::PayloadTypes pt,
-                               const char * nxeString)
+SDPMediaFormat::SDPMediaFormat(const OpalMediaFormat & fmt)
   : mediaFormat(fmt)
-  , payloadType(pt)
+  , payloadType(fmt.GetPayloadType())
   , clockRate(fmt.GetClockRate())
   , encodingName(fmt.GetEncodingName())
-  , nteSet(PTrue)
-  , nseSet(PTrue)
 {
-  if (nxeString != NULL) {
-    if (encodingName *= "nse")
-      AddNSEString(nxeString);
-    else
-      AddNTEString(nxeString);
-  }
   if (fmt.GetMediaType() == OpalMediaType::Audio()) 
     parameters = PString(PString::Unsigned, fmt.GetOptionInteger(OpalAudioFormat::ChannelsOption()));
 }
@@ -194,20 +176,6 @@ void SDPMediaFormat::SetFMTP(const PString & str)
   if (str.IsEmpty())
     return;
 
-  if (encodingName == OpalRFC2833.GetEncodingName()) {
-    nteSet.RemoveAll();
-    AddNTEString(str);
-    return;
-  }
-
-#if OPAL_T38_CAPABILITY
-  else if (encodingName == OpalCiscoNSE.GetEncodingName()) {
-    nseSet.RemoveAll();
-    AddNSEString(str);
-    return;
-  }
-#endif
-
   fmtp = str;
   if (GetMediaFormat().IsEmpty()) // Use GetMediaFormat() to force creation of member
     return;
@@ -229,10 +197,12 @@ void SDPMediaFormat::SetFMTP(const PString & str)
   // Parse the string for option names and values OPT=VAL;OPT=VAL
   PINDEX sep1prev = 0;
   do {
+    // find the next separator (' ' or ';')
     PINDEX sep1next = str.Find(sep, sep1prev);
     if (sep1next == P_MAX_INDEX)
       sep1next--; // Implicit assumption string is not a couple of gigabytes long ...
 
+    // find the next '='. If past the next separator, then ignore it
     PINDEX sep2pos = str.Find('=', sep1prev);
     if (sep2pos > sep1next)
       sep2pos = sep1next;
@@ -258,6 +228,12 @@ void SDPMediaFormat::SetFMTP(const PString & str)
         if (str.GetLength() % 2 != 0)
           value = value.Trim();
       } else {
+        // for non-octet string parameters, check for mixed separators
+        PINDEX brokenSep = str.Find(' ', sep2pos);
+        if (brokenSep < sep1next) {
+          sep1next = brokenSep;
+          value = str(sep2pos+1, sep1next-1);
+        }
         value = value.Trim();
         if (value.IsEmpty())
           value = "1"; // Assume it is a boolean
@@ -274,14 +250,6 @@ void SDPMediaFormat::SetFMTP(const PString & str)
 
 PString SDPMediaFormat::GetFMTP() const
 {
-  if (encodingName == OpalRFC2833.GetEncodingName())
-    return GetNTEString();
-
-#if OPAL_T38_CAPABILITY
-  if (encodingName == OpalCiscoNSE.GetEncodingName())
-    return GetNSEString();
-#endif
-
   if (GetMediaFormat().IsEmpty()) // Use GetMediaFormat() to force creation of member
     return fmtp;
 
@@ -306,90 +274,9 @@ PString SDPMediaFormat::GetFMTP() const
 }
 
 
-PString SDPMediaFormat::GetNTEString() const
-{
-  return GetNXEString(nteSet);
-}
-
-void SDPMediaFormat::AddNTEString(const PString & str)
-{
-  AddNXEString(nteSet, str);
-}
-
-void SDPMediaFormat::AddNTEToken(const PString & ostr)
-{
-  AddNXEToken(nteSet, ostr);
-}
-
-PString SDPMediaFormat::GetNSEString() const
-{
-  return GetNXEString(nseSet);
-}
-
-void SDPMediaFormat::AddNSEString(const PString & str)
-{
-  AddNXEString(nseSet, str);
-}
-
-void SDPMediaFormat::AddNSEToken(const PString & ostr)
-{
-  AddNXEToken(nseSet, ostr);
-}
-
-PString SDPMediaFormat::GetNXEString(POrdinalSet & nxeSet) const
-{
-  PString str;
-  PINDEX i = 0;
-  while (i < 255) {
-    if (!nxeSet.Contains(POrdinalKey(i)))
-      i++;
-    else {
-      PINDEX start = i++;
-      while (nxeSet.Contains(POrdinalKey(i)))
-        i++;
-      if (!str.IsEmpty())
-        str += ",";
-      str += PString(PString::Unsigned, start);
-      if (i > start+1)
-        str += PString('-') + PString(PString::Unsigned, i-1);
-    }
-  }
-
-  return str;
-}
-
-
-void SDPMediaFormat::AddNXEString(POrdinalSet & nxeSet, const PString & str)
-{
-  PStringArray tokens = str.Tokenise(",", PFalse);
-  PINDEX i;
-  for (i = 0; i < tokens.GetSize(); i++)
-    AddNXEToken(nxeSet, tokens[i]);
-}
-
-
-void SDPMediaFormat::AddNXEToken(POrdinalSet & nxeSet, const PString & ostr)
-{
-  PString str = ostr.Trim();
-  if (str[0] == ',')
-    str = str.Mid(1);
-  if (str.Right(1) == ",")
-    str = str.Left(str.GetLength()-1);
-  PINDEX pos = str.Find('-');
-  if (pos == P_MAX_INDEX)
-    nxeSet.Include(new POrdinalKey(str.AsInteger()));
-  else {
-    PINDEX from = str.Left(pos).AsInteger();
-    PINDEX to   = str.Mid(pos+1).AsInteger();
-    while (from <= to)
-      nxeSet.Include(new POrdinalKey(from++));
-  }
-}
-
-
 void SDPMediaFormat::PrintOn(ostream & strm) const
 {
-  PAssert(!encodingName.IsEmpty(), "SDPAudioMediaFormat encoding name is empty");
+  PAssert(!encodingName.IsEmpty(), "SDPMediaFormat encoding name is empty");
   mediaFormat.ToCustomisedOptions();
 
   PINDEX i;
@@ -419,11 +306,19 @@ void SDPMediaFormat::PrintOn(ostream & strm) const
   }
 }
 
+void SDPMediaFormat::UpdateMediaFormat() const
+{
+  mediaFormat = OpalMediaFormat(payloadType, clockRate, encodingName, "sip");
+  if (mediaFormat.IsEmpty())
+    mediaFormat = OpalMediaFormat(encodingName);
+}
 
 const OpalMediaFormat & SDPMediaFormat::GetMediaFormat() const
 {
   if (mediaFormat.IsEmpty()) {
-    mediaFormat = OpalMediaFormat(payloadType, clockRate, encodingName, "sip");
+
+    UpdateMediaFormat();
+
     PTRACE_IF(2, mediaFormat.IsEmpty(), "SDP\tCould not find media format for \""
               << encodingName << "\", pt=" << payloadType << ", clock=" << clockRate);
 
@@ -459,7 +354,7 @@ bool SDPMediaFormat::ToNormalisedOptions()
   if (mediaFormat.ToNormalisedOptions())
     return true;
 
-  PTRACE(2, "SDP\tCould not normalise format \"" << GetEncodingName() << "\", pt=" << GetPayloadType() << ", removing.");
+  PTRACE(2, "SDP\tCould not normalise format \"" << encodingName << "\", pt=" << payloadType << ", removing.");
   return false;
 }
 
@@ -476,18 +371,6 @@ void SDPMediaFormat::SetPacketTime(const PString & optionName, unsigned ptime)
 }
 
 
-PBoolean SDPMediaDescription::SetTransportAddress(const OpalTransportAddress &t)
-{
-  PIPSocket::Address ip;
-  WORD port = 0;
-  if (transportAddress.GetIpAndPort(ip, port)) {
-    transportAddress = OpalTransportAddress(t, port);
-    return PTrue;
-  }
-  return PFalse;
-}
-
-
 //////////////////////////////////////////////////////////////////////////////
 
 unsigned & SDPBandwidth::operator[](const PString & type)
@@ -534,11 +417,26 @@ SDPMediaDescription::SDPMediaDescription(const OpalTransportAddress & address)
   portCount = 0;
 }
 
+PBoolean SDPMediaDescription::SetTransportAddress(const OpalTransportAddress &t)
+{
+  PIPSocket::Address ip;
+  WORD port = 0;
+  if (transportAddress.GetIpAndPort(ip, port)) {
+    transportAddress = OpalTransportAddress(t, port);
+    return PTrue;
+  }
+  return PFalse;
+}
 
 bool SDPMediaDescription::Decode(const PStringArray & tokens)
 {
+  if (tokens.GetSize() < 3) {
+    PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
+    return false;
+  }
+
   // parse the media type
-  mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0]);
+  mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0], tokens[2]);
   if (mediaType.empty()) {
     PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
     return false;
@@ -588,16 +486,22 @@ bool SDPMediaDescription::Decode(const PStringArray & tokens)
         transportAddress = OpalTransportAddress(ip, (WORD)port);
   }
 
+  CreateSDPMediaFormats(tokens);
+
+  return PTrue;
+}
+
+void SDPMediaDescription::CreateSDPMediaFormats(const PStringArray & tokens)
+{
   // create the format list
   for (PINDEX i = 3; i < tokens.GetSize(); i++) {
     SDPMediaFormat * fmt = CreateSDPMediaFormat(tokens[i]);
-    if (fmt == NULL) {
+    if (fmt != NULL) 
+      formats.Append(fmt);
+    else {
       PTRACE(2, "SDP\tCannot create SDP media format for port " << tokens[i]);
     }
-    formats.Append(fmt);
   }
-
-  return PTrue;
 }
 
 
@@ -696,6 +600,7 @@ SDPMediaFormat * SDPMediaDescription::FindFormat(PString & params) const
   // extract the RTP payload type
   PINDEX pos = params.FindSpan("0123456789");
   if (pos == P_MAX_INDEX || isspace(params[pos])) {
+
     // find the format that matches the payload type
     RTP_DataFrame::PayloadTypes pt = (RTP_DataFrame::PayloadTypes)params.Left(pos).AsUnsigned();
     for (format = formats.begin(); format != formats.end(); ++format) {
@@ -706,9 +611,9 @@ SDPMediaFormat * SDPMediaDescription::FindFormat(PString & params) const
   else {
     // Check for it a format name
     pos = params.Find(' ');
-    PString fmtName = params.Left(pos);
+    PString encodingName = params.Left(pos);
     for (format = formats.begin(); format != formats.end(); ++format) {
-      if (format->GetEncodingName() == fmtName)
+      if (format->GetEncodingName() == encodingName)
         break;
     }
   }
@@ -876,7 +781,7 @@ void SDPMediaDescription::AddMediaFormat(const OpalMediaFormat & mediaFormat)
     }
   }
 
-  SDPMediaFormat * sdpFormat = new SDPMediaFormat(mediaFormat, payloadType);
+  SDPMediaFormat * sdpFormat = new SDPMediaFormat(mediaFormat);
 
   ProcessMediaOptions(*sdpFormat, mediaFormat);
 
@@ -1276,6 +1181,8 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
           case 'm' : // media name and transport address (mandatory)
             {
               if (currentMedia != NULL) {
+                PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
+                                                            << " '" << currentMedia->GetSDPMediaType() << "' formats");
                 if (!currentMedia->PostDecode())
                   ok = false;
               }
@@ -1288,7 +1195,8 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
               else
               {
                 // parse the media type
-                OpalMediaType mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0]);
+                PString mt = tokens[0].ToLower();
+                OpalMediaType mediaType = OpalMediaType::GetMediaTypeFromSDP(tokens[0], tokens[2]);
                 if (mediaType.empty()) {
                   PTRACE(1, "SDP\tUnknown SDP media type " << tokens[0]);
                   currentMedia = NULL;
@@ -1307,8 +1215,6 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
                     }
                     else if (currentMedia->Decode(tokens)) {
                       mediaDescriptions.Append(currentMedia);
-                      PTRACE(3, "SDP\tAdding media session with " << currentMedia->GetSDPMediaFormats().GetSize()
-                                                                  << ' ' << currentMedia->GetSDPMediaType() << " formats");
                     }
                     else {
                       delete currentMedia;
@@ -1328,6 +1234,8 @@ PBoolean SDPSessionDescription::Decode(const PString & str)
   }
 
   if (currentMedia != NULL) {
+    PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
+                                                << " '" << currentMedia->GetSDPMediaType() << "' formats");
     if (!currentMedia->PostDecode())
       ok = false;
   }
