@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision$
- * $Author$
- * $Date$
+ * $Revision: 21841 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-19 04:59:52 +0000 (Fri, 19 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -223,47 +223,62 @@ SIPURL::SIPURL(const PString & str, const char * defaultScheme)
 
 
 SIPURL::SIPURL(const PString & name,
-               const OpalTransportAddress & address,
+               const OpalTransportAddress & _address,
                WORD listenerPort)
 {
-  if (strncmp(name, "sip:", 4) == 0 || strncmp(name, "sips:", 5) == 0 || address.IsEmpty())
+  if (strncmp(name, "sip:", 4) == 0 || strncmp(name, "sips:", 5) == 0)
     Parse(name);
   else {
-    PIPSocket::Address ip;
-    WORD port;
-    if (address.GetIpAndPort(ip, port)) {
-      PString transProto;
-      WORD defaultPort = 5060;
+    OpalTransportAddress address(_address);
+    if (address.IsEmpty() && (name.Find('$') != P_MAX_INDEX)) 
+      address = name;
+    ParseAsAddress(name, address, listenerPort);
+  }
+}
 
-      PStringStream s;
-      s << "sip";
+SIPURL::SIPURL(const OpalTransportAddress & _address, WORD listenerPort)
+{
+  ParseAsAddress("", _address, listenerPort);
+}
+  
+void SIPURL::ParseAsAddress(const PString & name, const OpalTransportAddress & address, WORD listenerPort)
+{
+  PIPSocket::Address ip;
+  WORD port;
+  if (address.GetIpAndPort(ip, port)) {
+    PString transProto;
+    WORD defaultPort = 5060;
 
-      PCaselessString proto = address.Left(address.Find('$'));
-      if (proto == "tcps") {
-        defaultPort = 5061;
-        s << 's';
-        // use default tranport=UDP
-      }
-      else if (proto != "udp")
-        transProto = proto; // Typically "tcp"
-      // else use default UDP
+    PStringStream s;
+    s << "sip";
 
-      s << ':' << name << '@';
-      if (ip.GetVersion() == 6)
-        s << '[' << ip << ']';
-      else
-        s << ip;
-
-      if (listenerPort == 0)
-        listenerPort = port;
-      if (listenerPort != 0 && listenerPort != defaultPort)
-        s << ':' << listenerPort;
-
-      if (!transProto.IsEmpty())
-        s << ";transport=" << transProto;
-
-      Parse(s);
+    PCaselessString proto = address.GetProto();
+    if (proto == "tcps") {
+      defaultPort = 5061;
+      s << 's';
+      // use default tranport=UDP
     }
+    else if (proto != "udp")
+      transProto = proto; // Typically "tcp"
+    // else use default UDP
+
+    s << ':';
+    if (!name.IsEmpty())
+      s << name << '@';
+    if (ip.GetVersion() == 6)
+      s << '[' << ip << ']';
+    else
+      s << ip;
+
+    if (listenerPort == 0)
+      listenerPort = port;
+    if (listenerPort != 0 && listenerPort != defaultPort)
+      s << ':' << listenerPort;
+
+    if (!transProto.IsEmpty())
+      s << ";transport=" << transProto;
+
+    Parse(s);
   }
 }
 
@@ -285,7 +300,7 @@ PBoolean SIPURL::InternalParse(const char * cstr, const char * p_defaultScheme)
   if (startBracket == P_MAX_INDEX || endBracket == P_MAX_INDEX) {
     if (!PURL::InternalParse(cstr, defaultScheme))
       return false;
-  }
+    }
   else {
     // get the URI from between the angle brackets
     if (!PURL::InternalParse(str(startBracket+1, endBracket-1), defaultScheme))
@@ -300,7 +315,7 @@ PBoolean SIPURL::InternalParse(const char * cstr, const char * p_defaultScheme)
       // There are no double quotes around the display name, so take
       // everything before the start angle bracket
       displayName = str.Left(startBracket-1).Trim();
-    }
+      }
     else {
       // Trim quotes off
       displayName = str(startQuote+1, endQuote-1);
@@ -308,10 +323,10 @@ PBoolean SIPURL::InternalParse(const char * cstr, const char * p_defaultScheme)
       while ((backslash = displayName.Find('\\')) != P_MAX_INDEX)
         displayName.Delete(backslash, 1);
     }
-  }
+    }
 
   if (scheme == "sip" || scheme == "sips")
-    Recalculate();
+  Recalculate();
   else
     PURL::Parse("");
   return !IsEmpty();
@@ -358,6 +373,12 @@ PString SIPURL::AsQuotedString() const
     s << '"' << displayName << "\" ";
   s << '<' << AsString() << '>';
 
+  if (!fieldParameters.IsEmpty()) {
+    if (fieldParameters[0] != ';')
+      s << ';';
+    s << fieldParameters;
+  }
+
   return s;
 }
 
@@ -414,11 +435,11 @@ void SIPURL::Sanitise(UsageContext context)
     unsigned     contexts;
   } const SanitaryFields[] = {
     { "method",    (1<<RequestURI)|(1<<ToURI)|(1<<FromURI)|(1<<ContactURI)|(1<<RouteURI)|(1<<RegisterURI) },
-    { "maddr",     (1<<ToURI)|(1<<FromURI) },
-    { "ttl",       (1<<ToURI)|(1<<FromURI)|(1<<RouteURI)|(1<<RegisterURI) },
-    { "transport", (1<<ToURI)|(1<<FromURI) },
-    { "lr",        (1<<ToURI)|(1<<FromURI)|(1<<ContactURI)|(1<<RegisterURI) },
-    { "tag",       (1<<RequestURI)|(1<<ContactURI)|(1<<RouteURI)|(1<<ExternalURI)|(1<<RegisterURI) }
+    { "maddr",                     (1<<ToURI)|(1<<FromURI) },
+    { "ttl",                       (1<<ToURI)|(1<<FromURI)|(1<<RouteURI)|                (1<<RegisterURI) },
+    { "transport",                 (1<<ToURI)|(1<<FromURI) },
+    { "lr",                        (1<<ToURI)|(1<<FromURI)|(1<<ContactURI)|              (1<<RegisterURI) },
+    { "tag",       (1<<RequestURI)|                        (1<<ContactURI)|(1<<RouteURI)|(1<<RegisterURI)|(1<<ExternalURI) }
   };
 
   for (PINDEX i = 0; i < PARRAYSIZE(SanitaryFields); i++) {
@@ -466,8 +487,8 @@ PBoolean SIPURL::AdjustToDNS(PINDEX entry)
     return PFalse;
 
   // Adjust our host and port to what the DNS SRV record says
-  SetHostName(addrs[entry].address.AsString());
-  SetPort(addrs[entry].port);
+  SetHostName(addrs[entry].GetAddress().AsString());
+  SetPort(addrs[entry].GetPort());
   return PTrue;
 }
 #else
@@ -744,7 +765,14 @@ void SIPMIMEInfo::SetCSeq(const PString & v)
 
 PStringList SIPMIMEInfo::GetRoute() const
 {
-  return GetRouteList("Route");
+  return GetRouteList("Route", false);
+}
+
+
+void SIPMIMEInfo::SetRoute(const PString & v)
+{
+  if (!v.IsEmpty())
+    SetAt("Route",  v);
 }
 
 
@@ -755,9 +783,9 @@ void SIPMIMEInfo::SetRoute(const PStringList & v)
 }
 
 
-PStringList SIPMIMEInfo::GetRecordRoute() const
+PStringList SIPMIMEInfo::GetRecordRoute(bool reversed) const
 {
-  return GetRouteList("Record-Route");
+  return GetRouteList("Record-Route", reversed);
 }
 
 
@@ -768,7 +796,7 @@ void SIPMIMEInfo::SetRecordRoute(const PStringList & v)
 }
 
 
-PStringList SIPMIMEInfo::GetRouteList(const char * name) const
+PStringList SIPMIMEInfo::GetRouteList(const char * name, bool reversed) const
 {
   PStringList routeSet;
 
@@ -777,8 +805,13 @@ PStringList SIPMIMEInfo::GetRouteList(const char * name) const
   PINDEX right = 0;
   while ((left = s.Find('<', right)) != P_MAX_INDEX &&
          (right = s.Find('>', left)) != P_MAX_INDEX &&
-         (right - left) > 5)
-    routeSet.AppendString(s(left+1, right-1));
+         (right - left) > 5) {
+    PString * pstr = new PString(s(left+1, right-1));
+    if (reversed)
+      routeSet.InsertAt(0, pstr);
+    else
+      routeSet.Append(pstr);
+  }
 
   return routeSet;
 }
@@ -1097,6 +1130,21 @@ void SIPMIMEInfo::SetSIPETag(const PString & v)
 }
 
 
+PString SIPMIMEInfo::GetRequire() const
+{
+  return GetString("Require");  // no compact form
+}
+
+
+void SIPMIMEInfo::SetRequire(const PString & v, bool overwrite)
+{
+  if (overwrite || !Contains("Require"))
+    SetAt("Require",  v);        // no compact form
+  else
+    SetAt("Require",  GetString("Require") + ", " + v);
+}
+
+
 static bool LocateFieldParameter(const PString & fieldValue, const PString & paramName, PINDEX & start, PINDEX & end)
 {
   PINDEX semicolon = (PINDEX)-1;
@@ -1566,7 +1614,7 @@ SIP_PDU::SIP_PDU(const SIP_PDU & request,
   mime.SetCallID(requestMIME.GetCallID());
   mime.SetCSeq(requestMIME.GetCSeq());
   mime.SetVia(requestMIME.GetVia());
-  mime.SetRecordRoute(requestMIME.GetRecordRoute());
+  mime.SetRecordRoute(requestMIME.GetRecordRoute(false));
 
   /* Use extra parameter as redirection URL in case of 302 */
   if (code == SIP_PDU::Redirection_MovedTemporarily) {
@@ -1667,9 +1715,9 @@ void SIP_PDU::Construct(Methods meth,
   str << "SIP/" << versionMajor << '.' << versionMinor << '/'
       << via.Left(dollar).ToUpper() << ' ';
   PIPSocket::Address ip;
-  WORD port;
+  WORD port = 5060;
   if (via.GetIpAndPort(ip, port))
-    str << ip << ':' << port;
+    str << ip.AsString(true) << ':' << port;
   else
     str << via.Mid(dollar+1);
   str << ";branch=z9hG4bK" << branch << ";rport";
@@ -1682,32 +1730,36 @@ void SIP_PDU::Construct(Methods meth,
                         SIPConnection & connection,
                         const OpalTransport & transport)
 {
-  PStringList routeSet = connection.GetRouteSet();
   SIPEndPoint & endpoint = connection.GetEndPoint();
   PString localPartyName = connection.GetLocalPartyName();
+  PINDEX pos = localPartyName.Find('@');
+  if (pos != P_MAX_INDEX)
+    localPartyName = localPartyName.Left(pos);
   SIPURL contact = endpoint.GetContactURL(transport, localPartyName, SIPURL(connection.GetRemotePartyAddress()).GetHostName());
-  SIPURL via = endpoint.GetLocalURL(transport, localPartyName);
   contact.Sanitise(meth != Method_INVITE ? SIPURL::ContactURI : SIPURL::RouteURI);
   mime.SetContact(contact);
 
+  SIPURL via = endpoint.GetLocalURL(transport, localPartyName);
+
+  SIPDialogContext & dialog = connection.GetDialog();
   Construct(meth,
-            connection.GetRequestURI(),
-            connection.GetDialogTo(),
-            connection.GetDialogFrom(),
-            connection.GetToken(),
-            connection.GetNextCSeq(),
+            dialog.GetRequestURI(),
+            dialog.GetRemoteURI().AsQuotedString(),
+            dialog.GetLocalURI().AsQuotedString(),
+            dialog.GetCallID(),
+            dialog.GetNextCSeq(),
             via.GetHostAddress());
 
-  SetRoute(routeSet); // Possibly adjust the URI and the route
+  SetRoute(dialog.GetRouteSet()); // Possibly adjust the URI and the route
 }
 
 
-PBoolean SIP_PDU::SetRoute(const PStringList & set)
+bool SIP_PDU::SetRoute(const PStringList & set)
 {
   PStringList routeSet = set;
 
   if (routeSet.IsEmpty())
-    return PFalse;
+    return false;
 
   SIPURL firstRoute = routeSet.front();
   if (!firstRoute.GetParamVars().Contains("lr")) {
@@ -1720,7 +1772,19 @@ PBoolean SIP_PDU::SetRoute(const PStringList & set)
   }
 
   mime.SetRoute(routeSet);
-  return PTrue;
+  return true;
+}
+
+
+bool SIP_PDU::SetRoute(const SIPURL & proxy)
+{
+  if (proxy.IsEmpty())
+    return false;
+
+  PStringStream str;
+  str << "<sip:" << proxy.GetHostName() << ':'  << proxy.GetPort() << ";lr>";
+  mime.SetRoute(str);
+  return true;
 }
 
 
@@ -1785,14 +1849,14 @@ void SIP_PDU::AdjustVia(OpalTransport & transport)
 }
 
 
-bool SIP_PDU::SendResponse(OpalTransport & transport, StatusCodes code, const char * contact, const char * extra)
+bool SIP_PDU::SendResponse(OpalTransport & transport, StatusCodes code, SIPEndPoint * endpoint, const char * contact, const char * extra)
 {
   SIP_PDU response(*this, code, contact, extra);
-  return SendResponse(transport, response);
+  return SendResponse(transport, response, endpoint);
 }
 
 
-bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response)
+bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response, SIPEndPoint * endpoint)
 {
   OpalTransportAddress newAddress;
 
@@ -1855,6 +1919,14 @@ bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response)
     }
   }
 
+  if (endpoint != NULL && response.GetMIME().GetContact().IsEmpty()) {
+    SIPURL to = GetMIME().GetTo();
+    PString username = to.GetUserName();
+    SIPURL contact = endpoint->GetLocalURL(transport, username);
+    contact.Sanitise(SIPURL::ContactURI);
+    response.GetMIME().SetContact(contact);
+  }
+
   return response.Write(transport, newAddress);
 }
 
@@ -1874,7 +1946,7 @@ void SIP_PDU::PrintOn(ostream & strm) const
 PBoolean SIP_PDU::Read(OpalTransport & transport)
 {
   if (!transport.IsOpen()) {
-    PTRACE(1, "SIP\tAttempt to read PDU from closed tansport " << transport);
+    PTRACE(1, "SIP\tAttempt to read PDU from closed transport " << transport);
     return PFalse;
   }
 
@@ -2027,19 +2099,25 @@ PBoolean SIP_PDU::Write(OpalTransport & transport, const OpalTransportAddress &
   PWaitAndSignal mutex(transport.GetWriteMutex());
 
   if (!transport.IsOpen()) {
-    PTRACE(1, "SIP\tAttempt to write PDU to closed tansport " << transport);
+    PTRACE(1, "SIP\tAttempt to write PDU to closed transport " << transport);
     return PFalse;
   }
 
-  if (!remoteAddress.IsEmpty() && !transport.GetRemoteAddress().IsEquivalent(remoteAddress)) {
+  OpalTransportAddress oldRemoteAddress = transport.GetRemoteAddress();
+  if (!remoteAddress.IsEmpty() && !oldRemoteAddress.IsEquivalent(remoteAddress)) {
     if (!transport.SetRemoteAddress(remoteAddress)) {
-      PTRACE(1, "SIP\tCannot use remote address " << remoteAddress << " for tansport " << transport);
+      PTRACE(1, "SIP\tCannot use remote address " << remoteAddress << " for transport " << transport);
       return false;
     }
   }
 
-  if (!localInterface.IsEmpty() && transport.GetInterface() != localInterface)
-    transport.SetInterface(localInterface);
+  PString oldInterface = transport.GetInterface();
+  if (!localInterface.IsEmpty() && oldInterface != localInterface) {
+    if (!transport.SetInterface(localInterface)) {
+      PTRACE(1, "SIP\tCannot use local interface \"" << localInterface << "\" for transport " << transport);
+      return false;
+    }
+  }
 
   mime.SetCompactForm(false);
   PString strPDU = Build();
@@ -2077,11 +2155,13 @@ PBoolean SIP_PDU::Write(OpalTransport & transport, const OpalTransportAddress &
   }
 #endif
 
-  if (transport.WriteString(strPDU))
-    return PTrue;
+  bool ok = transport.WriteString(strPDU);
+  PTRACE_IF(1, !ok, "SIP\tPDU Write failed: " << transport.GetErrorText(PChannel::LastWriteError));
 
-  PTRACE(1, "SIP\tPDU Write failed: " << transport.GetErrorText(PChannel::LastWriteError));
-  return PFalse;
+  transport.SetInterface(oldInterface);
+  transport.SetRemoteAddress(oldRemoteAddress);
+
+  return ok;
 }
 
 
@@ -2144,6 +2224,141 @@ SDPSessionDescription * SIP_PDU::GetSDP()
 }
 
 
+////////////////////////////////////////////////////////////////////////////////////
+
+SIPDialogContext::SIPDialogContext()
+  : m_lastSentCSeq(0)
+  , m_lastReceivedCSeq(0)
+{
+  GenerateCallID();
+}
+
+
+void SIPDialogContext::GenerateCallID()
+{
+  m_callId = PGloballyUniqueID().AsString() + '@' + PIPSocket::GetHostName();
+}
+
+
+static void SetWithTag(const SIPURL & url, SIPURL & uri, PString & tag, bool generate)
+{
+  uri = url;
+
+  PString newTag = url.GetParamVars()("tag");
+  if (newTag.IsEmpty())
+    newTag = SIPMIMEInfo::ExtractFieldParameter(uri.GetFieldParameters(), "tag");
+  else
+    uri.SetParamVar("tag", PString::Empty());
+
+  if (!newTag.IsEmpty() && tag != newTag) {
+    PTRACE(4, "SIP\tUpdating dialog tag from \"" << tag << "\" to \"" << newTag << '"');
+    tag = newTag;
+  }
+
+  if (tag.IsEmpty() && generate)
+    tag = PGloballyUniqueID().AsString();
+
+  if (!tag.IsEmpty())
+    uri.SetFieldParameters("tag="+tag);
+}
+
+
+static bool SetWithTag(const PString & str, SIPURL & uri, PString & tag, bool generate)
+{
+  SIPURL url;
+  if (!url.Parse(str))
+    return false;
+
+  SetWithTag(url, uri, tag, generate);
+  return true;
+}
+
+
+void SIPDialogContext::SetLocalURI(const SIPURL & url)
+{
+  SetWithTag(url, m_localURI, m_localTag, true);
+}
+
+
+bool SIPDialogContext::SetLocalURI(const PString & uri)
+{
+  return SetWithTag(uri, m_localURI, m_localTag, true);
+}
+
+
+void SIPDialogContext::SetRemoteURI(const SIPURL & url)
+{
+  SetWithTag(url, m_remoteURI, m_remoteTag, false);
+}
+
+
+bool SIPDialogContext::SetRemoteURI(const PString & uri)
+{
+  return SetWithTag(uri, m_remoteURI, m_remoteTag, false);
+}
+
+
+void SIPDialogContext::UpdateRouteSet(const SIPURL & proxy)
+{
+  // Default routeSet if there is a proxy
+  if (m_routeSet.IsEmpty() && !proxy.IsEmpty()) {
+    PStringStream str;
+    str << "sip:" << proxy.GetHostName() << ':'  << proxy.GetPort() << ";lr";
+    m_routeSet += str;
+  }
+}
+
+
+void SIPDialogContext::Update(const SIP_PDU & pdu)
+{
+  const SIPMIMEInfo & mime = pdu.GetMIME();
+
+  if (m_routeSet.IsEmpty()) {
+    // get the route set from the Record-Route response field according to 12.1.2
+    // requests in a dialog do not modify the initial route set according to 12.2
+    m_routeSet = mime.GetRecordRoute(pdu.GetMethod() == SIP_PDU::NumMethods);
+  }
+
+  // Update request URI
+  if (pdu.GetMethod() != SIP_PDU::NumMethods || pdu.GetStatusCode()/100 == 2) {
+    PString contact = mime.GetContact();
+    if (!contact.IsEmpty()) {
+      m_requestURI.Parse(contact);
+      PTRACE(4, "SIP\tSet Request URI to " << m_requestURI);
+    }
+  }
+
+  /* Update the local/remote fields */
+  if (pdu.GetMethod() == SIP_PDU::NumMethods) {
+    SetRemoteURI(mime.GetTo());
+    SetLocalURI(mime.GetFrom());
+  }
+  else {
+    SetLocalURI(mime.GetTo()); // Will add a tag
+    SetRemoteURI(mime.GetFrom());
+  }
+}
+
+
+bool SIPDialogContext::IsDuplicateCSeq(unsigned requestCSeq)
+{
+  /* See if have had a mesage before and this one is older than the last,
+     with a check for server bugs where the sequence number changes
+     dramatically. if older then is a retransmission so return true and do
+     not process it. */
+  bool duplicate = m_lastReceivedCSeq != 0 && requestCSeq <= m_lastReceivedCSeq && (m_lastReceivedCSeq - requestCSeq) < 10;
+
+  PTRACE_IF(4, m_lastReceivedCSeq == 0, "SIP\tDialog initial sequence number " << requestCSeq);
+  PTRACE_IF(3, duplicate, "SIP\tReceived duplicate sequence number " << requestCSeq);
+  PTRACE_IF(2, !duplicate && requestCSeq != m_lastReceivedCSeq+1,
+            "SIP\tReceived unexpected sequence number " << requestCSeq << ", expecting " << m_lastReceivedCSeq+1);
+
+  m_lastReceivedCSeq = requestCSeq;
+
+  return duplicate;
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////////
 
 SIPTransaction::SIPTransaction(SIPEndPoint & ep,
@@ -2184,6 +2399,19 @@ void SIPTransaction::Construct(const PTimeInterval & minRetryTime, const PTimeIn
 }
 
 
+void SIPTransaction::Construct(Methods method, SIPDialogContext & dialog)
+{
+  SIP_PDU::Construct(method,
+                     dialog.GetRequestURI(),
+                     dialog.GetRemoteURI().AsQuotedString(),
+                     dialog.GetLocalURI().AsQuotedString(),
+                     dialog.GetCallID(),
+                     dialog.GetNextCSeq(),
+                     endpoint.GetLocalURL(transport).GetHostAddress());
+  SetRoute(dialog.GetRouteSet());
+}
+
+
 SIPTransaction::~SIPTransaction()
 {
   PTRACE_IF(1, state < Terminated_Success, "SIP\tDestroying transaction " << mime.GetCSeq() << " which is not yet terminated.");
@@ -2196,6 +2424,9 @@ PBoolean SIPTransaction::Start()
   if (state == Completed)
     return PTrue;
 
+  if (connection != NULL)
+    connection->OnStartTransaction(*this);
+
   endpoint.AddTransaction(this);
 
   if (state != NotStarted) {
@@ -2470,9 +2701,6 @@ void SIPTransaction::SetTerminated(States newState)
   state = newState;
   PTRACE(3, "SIP\tSet state " << StateNames[newState] << " for transaction " << mime.GetCSeq());
 
-  if (oldState != Completed)
-    completed.Signal();
-
   // Transaction failed, tell the endpoint
   switch (state) {
     case Terminated_Success :
@@ -2488,21 +2716,28 @@ void SIPTransaction::SetTerminated(States newState)
       if (connection != NULL)
         connection->OnTransactionFailed(*this);
   }
+
+  if (oldState != Completed)
+    completed.Signal();
+}
+
+
+void SIPTransaction::GenerateCallID(PString & callId)
+{
+  callId = PGloballyUniqueID().AsString() + '@' + PIPSocket::GetHostName();
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////////
 
-SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport, OpalRTPSessionManager * sm)
+SIPInvite::SIPInvite(SIPConnection & connection, OpalTransport & transport, const OpalRTPSessionManager & sm)
   : SIPTransaction(connection, transport, Method_INVITE)
+  , rtpSessions(sm)
 {
-  mime.SetDate() ;                             // now
+  mime.SetDate(); // now
   SetAllow(connection.GetEndPoint().GetAllowedMethods());
   mime.SetProductInfo(connection.GetEndPoint().GetUserAgent(), connection.GetProductInfo());
 
-  if (sm != NULL)
-    rtpSessions.CopyFromMaster(*sm);
-
   m_SDP = new SDPSessionDescription();
   if (!connection.OnSendSDP(false, rtpSessions, *m_SDP)) {
     delete m_SDP;
@@ -2527,9 +2762,10 @@ PBoolean SIPInvite::OnReceivedResponse(SIP_PDU & response)
       if (response.GetStatusCode() < 300) {
         // Need to update where the ACK goes to when have 2xx response
         SIPURL dest;
-        const PStringList & routeSet = connection->GetRouteSet();
+        SIPDialogContext & dialog = connection->GetDialog();
+        const PStringList & routeSet = dialog.GetRouteSet();
         if (routeSet.IsEmpty())
-          dest = connection->GetRequestURI();
+          dest = dialog.GetRequestURI();
         else
           dest = routeSet.front();
         m_remoteAddress = dest.GetHostAddress();
@@ -2575,7 +2811,7 @@ SIPRegister::Params::Params()
 
 SIPRegister::SIPRegister(SIPEndPoint & ep,
                          OpalTransport & trans,
-                         const PStringList & routeSet,
+                         const SIPURL & proxy,
                          const PString & id,
                          unsigned cseq,
                          const Params & params)
@@ -2596,90 +2832,138 @@ SIPRegister::SIPRegister(SIPEndPoint & ep,
   mime.SetContact(params.m_contactAddress);
   mime.SetExpires(params.m_expire);
 
-  SetRoute(routeSet);
+  SetAllow(ep.GetAllowedMethods());
+  SetRoute(proxy);
+}
+
+
+////////////////////////////////////////////////////////////////////////////////////
+
+static const char * const KnownEventPackage[SIPSubscribe::NumPredefinedPackages] = {
+  "message-summary",
+  "presence",
+  "dialog;sla;ma" // sla is the old version ma is the new for Line Appearance extension
+};
+
+SIPSubscribe::EventPackage::EventPackage(PredefinedPackages pkg)
+  : PCaselessString(pkg < NumPredefinedPackages ? KnownEventPackage[pkg] : "")
+{
+}
+
+
+bool SIPSubscribe::EventPackage::operator==(PredefinedPackages pkg) const
+{
+  return InternalCompare(0, P_MAX_INDEX, pkg < NumPredefinedPackages ? KnownEventPackage[pkg] : "") == EqualTo;
+}
+
+
+PObject::Comparison SIPSubscribe::EventPackage::InternalCompare(PINDEX offset, PINDEX length, const char * cstr) const
+{
+  // Special rules for comparing event package strings, only up to the ';', if present
+
+  for (;;) {
+    if (length-- == 0)
+      return EqualTo;
+    if (theArray[offset] == '\0' && cstr[offset] == '\0')
+      return EqualTo;
+    if (theArray[offset] == ';' || cstr[offset] == ';')
+      break;
+    Comparison c = PCaselessString::InternalCompare(offset, cstr[offset]);
+    if (c != EqualTo)
+      return c;
+    offset++;
+  }
+
+  const char * myIdPtr = strstr(theArray+offset, "id");
+  const char * theirIdPtr = strstr(cstr+offset, "id");
+  if (myIdPtr == NULL && theirIdPtr == NULL)
+    return EqualTo;
+
+  const char * myIdEnd = strchr(myIdPtr, ';');
+  PINDEX myIdLen = myIdEnd != NULL ? myIdEnd - myIdPtr : strlen(myIdPtr);
+
+  const char * theirIdEnd = strchr(theirIdPtr, ';');
+  PINDEX theirIdLen = theirIdEnd != NULL ? theirIdEnd - theirIdPtr : strlen(theirIdPtr);
+
+  if (myIdLen < theirIdLen)
+    return LessThan;
+
+  if (myIdLen > theirIdLen)
+    return GreaterThan;
+
+  return (Comparison)strncmp(myIdPtr, theirIdPtr, theirIdLen);
+}
+
+
+SIPSubscribe::Params::Params(PredefinedPackages pkg)
+  : m_eventPackage(pkg)
+  , m_expire(0)
+  , m_restoreTime(30)
+  , m_minRetryTime(PMaxTimeInterval)
+  , m_maxRetryTime(PMaxTimeInterval)
+{
+}
+
+
+SIPSubscribe::SIPSubscribe(SIPEndPoint & ep,
+                           OpalTransport & trans,
+                           SIPDialogContext & dialog,
+                           const Params & params)
+  : SIPTransaction(ep, trans)
+{
+  Construct(Method_SUBSCRIBE, dialog);
+
+  SIPURL contact;
+  if (params.m_contactAddress.IsEmpty()) {
+    // I have no idea why this is necessary, but it is the way OpenSIPS works ....
+    const SIPURL & userURI = params.m_eventPackage == SIPSubscribe::Dialog ? dialog.GetRemoteURI() : dialog.GetLocalURI();
+    contact = endpoint.GetLocalURL(trans, userURI.GetUserName());
+  }
+  else
+    contact = params.m_contactAddress;
+  contact.Sanitise(SIPURL::ContactURI);
+  mime.SetContact(contact);
+
+  mime.SetProductInfo(ep.GetUserAgent(), ep.GetProductInfo());
+  mime.SetEvent(params.m_eventPackage);
+  mime.SetExpires(params.m_expire);
+
+  SIPEventPackageHandler * packageHandler = SIPEventPackageFactory::CreateInstance(params.m_eventPackage);
+  if (packageHandler != NULL) {
+    mime.SetAccept(packageHandler->GetContentType());
+    delete packageHandler;
+  }
+
   SetAllow(ep.GetAllowedMethods());
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////////
 
-SIPSubscribe::Params::Params(PredefinedPackages pkg)
-  : m_eventPackage(SIPSubscribe::GetEventPackageName(pkg))
-  , m_expire(0)
-  , m_restoreTime(30)
-  , m_minRetryTime(PMaxTimeInterval)
-  , m_maxRetryTime(PMaxTimeInterval)
-{
-  switch (pkg) {
-    case SIPSubscribe::MessageSummary :
-      m_mimeType = "application/simple-message-summary";
-      break;
-
-    case SIPSubscribe::Presence :
-      m_mimeType = "application/pidf+xml";
-      break;
-
-    default :
-      break;
-  }
-}
-
-
-SIPSubscribe::PredefinedPackages SIPSubscribe::GetEventPackage(const PString & name)
-{
-  if (name *= "message-summary")
-    return SIPSubscribe::MessageSummary;
-  else if (name *= "presence")
-    return SIPSubscribe::Presence;
- 
-  return SIPSubscribe::NumPredefinedPackages;
-}
-
-
-PString SIPSubscribe::GetEventPackageName(PredefinedPackages pkg)
-{
-  static const char * const names[NumPredefinedPackages] = {
-    "message-summary",
-    "presence"
-  };
-  if (pkg < NumPredefinedPackages)
-    return names[pkg];
-  return PString::Empty();
-}
-
-
-SIPSubscribe::SIPSubscribe(SIPEndPoint & ep,
-                           OpalTransport & trans,
-                           const PStringList & routeSet,
-                           const PString & to,
-                           const PString & from,
-                           const PString & id,
-                           unsigned cseq,
-                           const Params & params)
+SIPNotify::SIPNotify(SIPEndPoint & ep,
+                     OpalTransport & trans,
+                     SIPDialogContext & dialog,
+                     const SIPEventPackage & eventPackage,
+                     const PString & state,
+                     const PString & body)
   : SIPTransaction(ep, trans)
 {
-  SIPURL targetAddress = params.m_targetAddress;
-  targetAddress.Sanitise(SIPURL::RequestURI);
+  Construct(Method_NOTIFY, dialog);
 
-  OpalTransportAddress viaAddress = ep.GetLocalURL(transport).GetHostAddress();
-  SIP_PDU::Construct(Method_SUBSCRIBE,
-                     targetAddress,
-                     to,
-                     from,
-                     id,
-                     cseq,
-                     viaAddress);
-
-  mime.SetProductInfo(ep.GetUserAgent(), ep.GetProductInfo());
-  SIPURL contact = endpoint.GetLocalURL(trans, SIPURL(from).GetUserName());
+  SIPURL contact = endpoint.GetLocalURL(trans, dialog.GetLocalURI().GetUserName());
   contact.Sanitise(SIPURL::ContactURI);
   mime.SetContact(contact);
-  mime.SetAccept(params.m_mimeType);
-  mime.SetEvent(params.m_eventPackage);
-  mime.SetExpires(params.m_expire);
+  mime.SetProductInfo(ep.GetUserAgent(), ep.GetProductInfo());
+  mime.SetEvent(eventPackage);
+  mime.SetSubscriptionState(state);
 
-  SetRoute(routeSet);
-  SetAllow(ep.GetAllowedMethods());
+  SIPEventPackageHandler * packageHandler = SIPEventPackageFactory::CreateInstance(eventPackage);
+  if (packageHandler != NULL) {
+    mime.SetContentType(packageHandler->GetContentType());
+    delete packageHandler;
+  }
+
+  entityBody = body;
 }
 
 
@@ -2687,16 +2971,14 @@ SIPSubscribe::SIPSubscribe(SIPEndPoint & ep,
 
 SIPPublish::SIPPublish(SIPEndPoint & ep,
                        OpalTransport & trans,
-                       const PStringList & routeSet,
-                       const SIPURL & targetAddress,
                        const PString & id,
                        const PString & sipIfMatch,
-                       const PString & body,
-                       unsigned expires)
+                       SIPSubscribe::Params & params,
+                       const PString & body)
   : SIPTransaction(ep, trans)
 {
+  SIPURL targetAddress = params.m_addressOfRecord;
   PString addrStr = targetAddress.AsQuotedString();
-  OpalTransportAddress viaAddress = ep.GetLocalURL(transport).GetHostAddress();
 
   SIP_PDU::Construct(Method_PUBLISH,
                      targetAddress,
@@ -2704,21 +2986,25 @@ SIPPublish::SIPPublish(SIPEndPoint & ep,
                      addrStr+";tag="+OpalGloballyUniqueID().AsString(),
                      id,
                      endpoint.GetNextCSeq(),
-                     viaAddress);
+                     ep.GetLocalURL(transport).GetHostAddress());
 
   mime.SetProductInfo(ep.GetUserAgent(), ep.GetProductInfo());
   SIPURL contact = endpoint.GetLocalURL(trans, targetAddress.GetUserName());
   contact.Sanitise(SIPURL::ContactURI);
   mime.SetContact(contact);
-  mime.SetExpires(expires);
+  mime.SetExpires(params.m_expire);
 
   if (!sipIfMatch.IsEmpty())
     mime.SetSIPIfMatch(sipIfMatch);
   
-  mime.SetEvent("presence");
-  mime.SetContentType("application/pidf+xml");
+  mime.SetEvent(params.m_eventPackage);
+  SIPEventPackageHandler * packageHandler = SIPEventPackageFactory::CreateInstance(params.m_eventPackage);
+  if (packageHandler != NULL) {
+    mime.SetContentType(packageHandler->GetContentType());
+    delete packageHandler;
+  }
 
-  SetRoute(routeSet);
+  SetRoute(SIPURL(params.m_agentAddress));
 
   if (!body.IsEmpty())
     entityBody = body;
@@ -2775,19 +3061,13 @@ SIPReferNotify::SIPReferNotify(SIPConnection & connection, OpalTransport & trans
 
 SIPMessage::SIPMessage(SIPEndPoint & ep,
                        OpalTransport & trans,
+                       const SIPURL & proxy,
                        const SIPURL & address,
-                       const PStringList & routeSet,
                        const PString & id,
                        const PString & body)
   : SIPTransaction(ep, trans)
 {
-  OpalTransportAddress viaAddress = ep.GetLocalURL(transport).GetHostAddress();
-    
-  // Build the correct From field
-  PString displayName = ep.GetDefaultDisplayName();
-  PString partyName = endpoint.GetRegisteredPartyName(address).AsString(); 
-
-  SIPURL myAddress("\"" + displayName + "\" <" + partyName + ">"); 
+  SIPURL myAddress = endpoint.GetRegisteredPartyName(address.GetHostName(), transport);
 
   SIP_PDU::Construct(Method_MESSAGE,
                      address.AsQuotedString(),
@@ -2795,9 +3075,9 @@ SIPMessage::SIPMessage(SIPEndPoint & ep,
                      myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
                      id,
                      endpoint.GetNextCSeq(),
-                     viaAddress);
+                     ep.GetLocalURL(transport).GetHostAddress());
   mime.SetContentType("text/plain;charset=UTF-8");
-  mime.SetRoute(routeSet);
+  SetRoute(proxy);
 
   entityBody = body;
 }
@@ -2810,23 +3090,13 @@ SIPPing::SIPPing(SIPEndPoint & ep,
                  const PString & body)
   : SIPTransaction(ep, trans)
 {
-  PString id = OpalGloballyUniqueID().AsString() + "@" + PIPSocket::GetHostName();
-  OpalTransportAddress viaAddress = ep.GetLocalURL(transport).GetHostAddress();
-    
-  // Build the correct From field
-  PString displayName = ep.GetDefaultDisplayName();
-  PString partyName = endpoint.GetRegisteredPartyName(address).AsString(); 
-
-  SIPURL myAddress("\"" + displayName + "\" <" + partyName + ">"); 
-  
   SIP_PDU::Construct(Method_PING,
                      address.AsQuotedString(),
                      address.AsQuotedString(),
-                     // myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
                      "sip:"+address.GetUserName()+"@"+address.GetHostName(),
-                     id,
+                     OpalGloballyUniqueID().AsString() + "@" + PIPSocket::GetHostName(),
                      endpoint.GetNextCSeq(),
-                     viaAddress);
+                     ep.GetLocalURL(transport).GetHostAddress());
   mime.SetContentType("text/plain;charset=UTF-8");
 
   entityBody = body;
@@ -2873,26 +3143,16 @@ SIPOptions::SIPOptions(SIPEndPoint & ep,
                        const SIPURL & address)
   : SIPTransaction(ep, trans)
 {
-  PString requestURI;
-  PString hosturl;
-  PString id = OpalGloballyUniqueID().AsString() + "@" + PIPSocket::GetHostName();
-  OpalTransportAddress viaAddress = ep.GetLocalURL(transport).GetHostAddress();
-    
   // Build the correct From field
-  PString displayName = ep.GetDefaultDisplayName();
-  PString partyName = endpoint.GetRegisteredPartyName(SIPURL(address).GetHostName()).AsString();
+  SIPURL myAddress = endpoint.GetRegisteredPartyName(address.GetHostName(), transport);
 
-  SIPURL myAddress("\"" + displayName + "\" <" + partyName + ">"); 
-
-  requestURI = "sip:" + address.AsQuotedString();
-  
   SIP_PDU::Construct(Method_OPTIONS,
-                     requestURI,
+                     address,
                      address.AsQuotedString(),
                      myAddress.AsQuotedString()+";tag="+OpalGloballyUniqueID().AsString(),
-                     id,
+                     OpalGloballyUniqueID().AsString() + "@" + PIPSocket::GetHostName(),
                      endpoint.GetNextCSeq(),
-                     viaAddress);
+                     ep.GetLocalURL(transport).GetHostAddress());
   mime.SetAccept("application/sdp, application/media_control+xml, application/dtmf, application/dtmf-relay");
 
   SetAllow(ep.GetAllowedMethods());
